package org.objectweb.telosys.admin;

import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.uil.screenmap.ScreenConventions;
import org.objectweb.telosys.uil.screenmap.ScreenDefConfig;
import org.objectweb.telosys.uil.screenmap.ScreenDefRootDir;
import org.objectweb.telosys.uil.screenmap.ScreenDefinition;
import org.objectweb.telosys.uil.screenmap.ScreenDefinitions;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;

/**
 *  
 */

public class ConsoleScreens extends Console
{
    //-----------------------------------------------------------------------------
    private static void printConfig(PrintWriter out, ScreenDefConfig config )
    {
        if ( null == config )
        {
            out.println("ERROR : ScreenDefConfig is null !");
            return ;
        }
        openTable( out );
        printTR(out, "screen_map_url", config.getScreenMapUrl() );
        printTR(out, "default_action", config.getDefaultAction() );
        printTR(out, "base_language", config.getBaseLanguage() );
        printTR(out, "all_languages", config.getAllLanguages() );
        printTR(out, "ajax_request_url", config.getAjaxRequestUrl() );
        printTR(out, "telosys_jar_file", config.getTelosysJarFile() );
        printTR(out, "framework_javascript_file", config.getFrameworkJavascriptFile() );
        printTR(out, "framework_css_dir", config.getFrameworkCSSDir() );
        printTR(out, "framework css file", config.getFrameworkCSSFile() );
        closeTable( out );
    }
    
    //-----------------------------------------------------------------------------
    private static void printConventions(PrintWriter out, ScreenConventions conv )
    {
        if ( null == conv )
        {
            out.println("ERROR : ScreenConventions is null !");
            return ;
        }
        openTable( out );
        printTR(out, "template ", conv.getTemplate() );
        printTR(out, "body ",     conv.getBody() );
        printTR(out, "script ",   conv.getScript() );
        printTR(out, "css ",      conv.getCss() );
        
        printTR(out, "contextid ",    conv.getContextId() );
        printTR(out, "contextname",   conv.getContextName() );
        printTR(out, "contextaction", conv.getContextAction() );
        
        closeTable( out );
    }
    
    
    //-----------------------------------------------------------------------------
    private static void printRootDir(PrintWriter out, ScreenDefRootDir rootdir )
    {
        if ( null == rootdir )
        {
            out.println("ERROR : root directory is null !");
            return ;
        }
        openTable( out );
        printTR(out, "body root dir", rootdir.getBodyRootDir() ) ;
        printTR(out, "css root dir", rootdir.getCssRootDir() ) ;
        printTR(out, "images root dir", rootdir.getImagesRootDir() ) ;
        printTR(out, "page root dir", rootdir.getPageRootDir() ) ;
        printTR(out, "script root dir", rootdir.getScriptRootDir() ) ;
        printTR(out, "template root dir", rootdir.getTemplateRootDir() ) ;
        closeTable( out );
    }
    //-----------------------------------------------------------------------------
    private static void printScreensDefinition(PrintWriter out, ScreenDefinition def )
    {
        if ( null == def )
        {
            printTR(out, "<b>ERROR</b> ", "screen definition is null !" ) ;
        }
        else
        {
	        printTR(out, "<b>SCREEN NAME</b> ", "<b>" + def.getScreenName() + "</b>") ;
	        
	        printTR(out, "html target", def.getScreenTarget(ScreenMapConst.HTML_SCREEN_TYPE ) ) ;
	        printTR(out, "html body",   def.getScreenBody(ScreenMapConst.HTML_SCREEN_TYPE ) ) ;
	        printTR(out, "html script", def.getScreenScript(ScreenMapConst.HTML_SCREEN_TYPE) );
	        printTR(out, "html css",    def.getScreenCss(ScreenMapConst.HTML_SCREEN_TYPE) );
	
	        printTR(out, "xul target", def.getScreenTarget(ScreenMapConst.XUL_SCREEN_TYPE ) ) ;
	        printTR(out, "xul body",   def.getScreenBody(ScreenMapConst.XUL_SCREEN_TYPE ) ) ;
	        printTR(out, "xul script", def.getScreenScript(ScreenMapConst.XUL_SCREEN_TYPE) );
	        printTR(out, "xul css",    def.getScreenCss(ScreenMapConst.XUL_SCREEN_TYPE) );
	        
	        printTR(out, "Context name",   def.getScreenContextName() );
	        printTR(out, "Context id",     "" + def.getScreenContextId() );
	        printTR(out, "Context action", def.getScreenContextAction() );
        }
        printTR(out, "&nbsp;", "&nbsp;") ;
    }
    //-----------------------------------------------------------------------------
    private static void printScreensDefinitions(PrintWriter out, ScreenDefinitions screenDefinitions )
    {
        openTable( out );

//        Iterator iter = screenDefinitions.getScreenNames() ;
//        while ( iter.hasNext() )
//        {
//            String screenName = (String) iter.next();
//            ScreenDefinition def = screenDefinitions.getScreenDefinition(screenName);
//            printScreensDefinition(out, def );
//        }
        String[] screenNames = screenDefinitions.getScreenNamesArray() ;
        
        java.util.Arrays.sort(screenNames);

        for ( int i = 0 ; i < screenNames.length ; i++ )
        {
            String screenName = screenNames[i];
            ScreenDefinition def = screenDefinitions.getScreenDefinition(screenName);
            printScreensDefinition(out, def );
        }
        
        closeTable( out );
    }
    //-----------------------------------------------------------------------------
    protected static void print(PrintWriter out, HttpServletRequest request)
    {
        ScreenDefinitions screenDefinitions = TelosysUIL.getScreenDefinitions();

        out.println("<b><u> Screens configuration : </u></b>");
        out.println("<br/>");
        printConfig( out, screenDefinitions.getConfig() ) ;
        
        out.println("<br/>");

        out.println("<b><u> Root directories for HTML screens : </u></b>");
        out.println("<br/>");
        printRootDir( out, screenDefinitions.getHtmlRootDir() );
        
        out.println("<br/>");

        out.println("<b><u> Root directories for XUL screens : </u></b>");
        out.println("<br/>");
        printRootDir( out, screenDefinitions.getXulRootDir() );
        
        out.println("<br/>");

        out.println("<b><u> Screens conventions : </u></b>");
        out.println("<br/>");
        printConventions(out, screenDefinitions.getScreenConventions() );

        out.println("<br/>");

        out.println("<b><u> Screens definitions : </u></b>");
        out.println("<br/>");
        printScreensDefinitions( out, screenDefinitions );
        
        printBack(out, request);
    }
}