package org.objectweb.telosys.common;

/**
 * ApplicationLogger wrapper to be secure when calling external log services<br>
 * ( just manage potential exceptions )
 * 
 * @author Laurent GUERIN
 *
 */
public class ApplicationLoggerFacade // implements IApplicationLogger
{

    private IApplicationLogger _appLogger = null ;
    
    //----------------------------------------------------------------------------------------
    /**
     * No constructor without parameters
     */
    private ApplicationLoggerFacade() 
    {
    }
    
    //----------------------------------------------------------------------------------------
    /**
     * Constructs an ApplicationLogger wrapper
     * @param logger
     */
    protected ApplicationLoggerFacade(IApplicationLogger logger) throws TelosysException 
    {
        super();
        if ( logger != null )
        {
            _appLogger = logger;
        }
        else
        {
            throw new TelosysException ("Cannot create ApplicationLoggerFacade : logger param is null ");
        }
    }
    
    //----------------------------------------------------------------------------------------
    private void reportError(String s, Throwable ex)
    {
        // Risk of infinite loop => Do not re-call Telosys.xxx method ! 
        // => use only the Console log writer 
        Console.error("Exception in ApplicationLoger." + s + " : " + ex.getClass() + " - " + ex.getMessage() );
    }
    
    //----------------------------------------------------------------------------------------
    /**
     * Call the application logger "trace" method
     * @param sClassName
     * @param sMessage
     */
    protected void trace(String sClassName, String sMessage)
    {
        try
        {
            _appLogger.trace(sClassName, sMessage);
        } catch (Throwable e)
        {
            reportError("trace",e);
        }
    }

    //----------------------------------------------------------------------------------------
    /**
     * Call the application logger "debug" method
     * @param sClassName
     * @param sMessage
     */
    protected void debug(String sClassName, String sMessage)
    {
        try
        {
            _appLogger.debug(sClassName, sMessage);
        } catch (Throwable e)
        {
            reportError("debug",e);
        }
    }

    //----------------------------------------------------------------------------------------
    /**
     * Call the application logger "info" method
     * @param sClassName
     * @param sMessage
     */
    protected void info(String sClassName, String sMessage)
    {
        try
        {
            _appLogger.info(sClassName, sMessage);
        } catch (Throwable e)
        {
            reportError("info",e);
        }
    }

    //----------------------------------------------------------------------------------------
    /**
     * Call the application logger "warn" method
     * @param sClassName
     * @param sMessage
     */
    protected void warn(String sClassName, String sMessage)
    {
        try
        {
            _appLogger.warn(sClassName, sMessage);
        } catch (Throwable e)
        {
            reportError("warn",e);
        }
    }

    //----------------------------------------------------------------------------------------
    /**
     * Call the application logger "error" method
     * @param sClassName
     * @param sMessage
     */
    protected void error(String sClassName, String sMessage)
    {
        try
        {
            _appLogger.error(sClassName, sMessage);
        } catch (Throwable e)
        {
            reportError("error",e);
        }
    }

    //----------------------------------------------------------------------------------------
    /**
     * Call the application logger "error" method
     * @param sClassName
     * @param exception
     */
    protected void error(String sClassName, Throwable exception)
    {
        try
        {
            _appLogger.error(sClassName, exception);
        } catch (Throwable e)
        {
            reportError("error",e);
        }
    }
}
