package org.objectweb.telosys.common;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.util.web.WebUtil;

/**
 * Local resource implementation of the View interface <br>
 * To use the resource located at the given path as a standard "MVC view"
 * 
 * @author Laurent GUERIN
 *  
 */
public class ResourceView extends TelosysObject implements View
{
    private String _sTarget = null;

    private ResourceView() // no default constructor
    {
    }

    /**
     * Constructs a new "resource view" with the given target path
     * @param sPath the path of the resource ( e.g. "/mydir/test/MyPage.jsp" )
     */
    public ResourceView(String sPath) //throws TelosysException
    {
        _sTarget = sPath;
    }
    
    //---------------------------------------------------------------------------
    public String toString() // v 1.0.5
    {
        return "ResourceView : '" + _sTarget + "'" ;        
    }

//  removed in v 1.0.5
//    /*
//     * (non-Javadoc)
//     * 
//     * @see org.objectweb.telosys.common.View#getTarget()
//     */
//    public String getTarget()
//    {
//        return _sTarget;
//    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.common.View#generate()
     */
    public void generate(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws TelosysException
    {
        //--- Try to forward to the Target
        trace("Forward to " + _sTarget + " ... ");
        boolean bForwardOk = false;
        String sMsgForwardError = "Cannot forward to target '" + _sTarget + "' ";
        try
        {
            bForwardOk = WebUtil.forward(_sTarget, httpRequest, httpResponse);
        } catch (ServletException e)
        {
            throw new TelosysException(sMsgForwardError + " (ServletException) ", e);
        } catch (IOException e)
        {
            throw new TelosysException(sMsgForwardError + " (IOException) ", e);
        }
        if (!bForwardOk)
        {
            throw new TelosysException(sMsgForwardError + " (invalid target) ");
        }
    }
}