package org.objectweb.telosys.common;

import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.env.ScreenContextManager;
import org.objectweb.telosys.uil.RequestEnv;
import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.uil.screenmap.ScreenDefinitions;
import org.objectweb.telosys.uil.screenmap.ScreenMap;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;
import org.objectweb.telosys.util.web.WebUtil;

/**
 * ScreenMap implementation of the View interface<br>
 * To use a screenmap as a standard "MVC view" in a "navigation service"
 *  
 * @author Laurent GUERIN
 *  
 */
public class ScreenView extends TelosysObject implements View
{

    private String    _sName      = null;

    private String    _sType      = null;

    private int       _iScreenContextId     = 0;
    
    private String    _sScreenContextName   = null ;

    private String    _sScreenContextAction = null;

    //--- removed in v 1.0.5
    //private ScreenMap _screenMap  = null; 

    //private String    _sTarget    = null;

    //---------------------------------------------------------------------------
    private ScreenView() // no default constructor
    {

    }

    //---------------------------------------------------------------------------
    /**
     * ScreenMap View constructor
     * @param sName : the name of the screen map ( name defined in "screens.xml" )
     * @param iScreenContextId : the id of the ScreenContext associated with the screen map 
     * @throws TelosysException
     */
    public ScreenView(String sName, int iScreenContextId) throws TelosysException
    {
        _sName = sName;
        _sType = TelosysUIL.getDefaultScreenType();
        _iScreenContextId   = iScreenContextId;
        _sScreenContextName = null ;
        //initView(); // removed in v 1.0.5
    }

    //---------------------------------------------------------------------------
    /**
     * ScreenMap View constructor
     * @param sName : the name of the screen map ( name defined in "screens.xml" )
     * @param sType : the type of the screen map ( "html" or "xul" )
     * @param iScreenContextId : the id of the ScreenContext associated with the screen map 
     * @throws TelosysException
     */
    public ScreenView(String sName, String sType, int iScreenContextId) throws TelosysException
    {
        _sName = sName;
        _sType = sType;
        _iScreenContextId   = iScreenContextId;
        _sScreenContextName = null ;
        //initView(); // removed in v 1.0.5
    }

    //---------------------------------------------------------------------------
    /**
     * ScreenMap View constructor
     * @param sName : the name of the screen map ( name defined in "screens.xml" )
     * @param iScreenContextId : the id of the ScreenContext associated with the screen map 
     * @param sScreenContextName : the name of the ScreenContext associated with the screen map 
     * @throws TelosysException
     */
    public ScreenView(String sName, int iScreenContextId, String sScreenContextName ) throws TelosysException
    {
        _sName = sName;
        _sType = TelosysUIL.getDefaultScreenType();
        _iScreenContextId   = iScreenContextId ;
        _sScreenContextName = sScreenContextName ;
        //initView(); // removed in v 1.0.5
    }
    
    //---------------------------------------------------------------------------
    /**
     * ScreenMap View constructor
     * @param sName : the name of the screen map ( name defined in "screens.xml" )
     * @param sType : the type of the screen map ( "html" or "xul" )
     * @param iScreenContextId : the id of the ScreenContext associated with the screen map 
     * @param sScreenContextName : the name of the ScreenContext associated with the screen map 
     * @throws TelosysException
     */
    public ScreenView(String sName, String sType, int iScreenContextId, String sScreenContextName ) throws TelosysException
    {
        _sName = sName ;
        _sType = sType ;
        _iScreenContextId   = iScreenContextId ;
        _sScreenContextName = sScreenContextName ;
        // initView(); // removed in v 1.0.5
    }

//    //---------------------------------------------------------------------------
//    private void initView() throws TelosysException
//    {
////        //--- Get the screen definitions
////        ScreenDefinitions screenDefinitions = TelosysUIL.getScreenDefinitions();
////        if (screenDefinitions == null)
////        {
////            throw new TelosysException("Screens definitions are not initialized ! ");
////        }
//
////        //--- Get the screen map for the current screen
////        // _screenMap = screenDefinitions.getScreenMap(_sName, _sType, _iScreenContextId, _sScreenContextName);
////        _screenMap = screenDefinitions.getScreenMap(_sName, _sType, _iScreenContextId, _sScreenContextName, null ); // v 1.0.5
////        if (_screenMap == null)
////        {
////            String sMsg = "Cannot get the screen map : getScreenMap(" + _sName + "," + _sType + "," + _iScreenContextId + ")";
////            throw new TelosysException(sMsg);
////        }
//
////        //--- Retrieve the main target for the screen map
////        _sTarget = _screenMap.getTargetPath();
////        if (_sTarget == null)
////        {
////            String sMsg = "No target for the screen : " + _sName + " ( type = " + _sType + " )";
////            throw new TelosysException(sMsg);
////        }
//    }

    //---------------------------------------------------------------------------
    /**
     * Returns the name of the ScreenMap ( logical name defined in "screens.xml" or by convention )
     * @return
     */
    public String getName()
    {
        return _sName;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the type of the ScreenMap ( e.g. "html", "xul" )
     * @return
     */
    public String getType()
    {
        return _sType;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the ScreenContext ID 
     * @return
     * @deprecated use getScreenContextId
     */
    public int getId()
    {
        return _iScreenContextId ;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the ScreenContext ID 
     * @return
     */
    public int getScreenContextId()
    {
        return _iScreenContextId ;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the ScreenContext name (can be null)
     * @return 
     */
    public String getScreenContextName()
    {
        return _sScreenContextName ;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the ScreenContext action
     * @return
     */
    public String getAction()
    {
        return _sScreenContextAction;
    }

//    //---------------------------------------------------------------------------
//    public ScreenMap getScreenMap()
//    {
//        return _screenMap;
//    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() // v 1.0.5
    {
        return "ScreenView : " + _sName + "-" + _sType + "/" 
        	+ _iScreenContextId + "-" + _sScreenContextName + ":" + _sScreenContextAction ;        
    }
    //---------------------------------------------------------------------------
    /**
     * Try to get the ScreenMap instance corresponding to the given parameters
     * @param servletContext
     * @return
     * @throws TelosysException
     * @since 1.0.5
     */
    private ScreenMap getScreenMap( ServletContext servletContext ) throws TelosysException
    {
        //--- Get the screen definitions
        ScreenDefinitions screenDefinitions = TelosysUIL.getScreenDefinitions();
        if (screenDefinitions == null)
        {
            throw new TelosysException("Screens definitions are not initialized ! ");
        }

        //--- Get the screen map for this VIEW
        //ScreenMap screenMap = screenDefinitions.getScreenMap(_sName, _sType, _iScreenContextId, _sScreenContextName, null ); // v 1.0.5
        ScreenMap screenMap = screenDefinitions.getScreenMap(servletContext, _sName, _sType, _iScreenContextId, _sScreenContextName, null ); // v 1.0.5
        if ( screenMap == null)
        {
            String sMsg = "Cannot get the screen map : getScreenMap(" + _sName + "," + _sType + "," + _iScreenContextId + ")";
            throw new TelosysException(sMsg);
        }
        return screenMap ;
    }

    //---------------------------------------------------------------------------
// removed in v 1.0.5
//    /*
//     * (non-Javadoc)
//     * 
//     * @see org.objectweb.telosys.common.View#getTarget()
//     */
//    public String getTarget()
//    {
//        return _sTarget;
//    }

    //---------------------------------------------------------------------------
    private void exposeScreenContext(HttpServletRequest httpRequest) 
    {
        //--- Try to find the ScreenContext associated with this view
        ScreenContext screenContext = null;
        try
        {
            if ( _sScreenContextName != null )
            {
                //--- Find ScreenContext by ID and NAME
                screenContext = ScreenContextManager.getScreenContextForName(httpRequest, _iScreenContextId, _sScreenContextName);                
            }
            else
            {
                //--- Find ScreenContext only by ID
                screenContext = ScreenContextManager.getScreenContext(httpRequest, _iScreenContextId);
            }
        } catch (TelosysException e)
        {
            // ScreenContext not accessible (not realy an error in this case)
            return ;
        }
        if (screenContext != null)
        {
            //--- Expose the Screen Context  
            RequestEnv.setEnv(httpRequest, screenContext, RequestEnv.EXPOSE_DATA );
        }
    }
    //---------------------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.common.View#generate()
     */
    public void generate(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws TelosysException
    {
        HttpSession session = httpRequest.getSession(); // get (or create) the http session 
        if ( null == session )
        {
            throw new TelosysException("Cannot get HttpSession from request");
        }
        ServletContext servletContext = session.getServletContext();
        if ( null == servletContext )
        {
            throw new TelosysException("Cannot get ServletContext from session");
        }

        //--- Get the ScreenMap for this view 
        ScreenMap screenMap = getScreenMap( servletContext ) ;
        //if (_screenMap == null)
        if ( null == screenMap )
        {
            String sMsg = "No screen map for this view : " + _sName + " ( id = " + _iScreenContextId + " type = " + _sType + " )";
            throw new TelosysException(sMsg);
        }

        //--- Retrieve the main target for the screen map
        String sTarget = screenMap.getTargetPath();
        if ( null == sTarget )
        {
            String sMsg = "No target for this ScreenMap : " + _sName + " ( type = " + _sType + " )";
            throw new TelosysException(sMsg);
        }
        
        //--- Set the Screen Map object as request attribute
        //httpRequest.setAttribute(ScreenMapConst.SCREEN_MAP, _screenMap);
        httpRequest.setAttribute(ScreenMapConst.SCREEN_MAP, screenMap); // v 1.0.5
        trace("Screen map set as request attribute");

        //--- Set the Screen Context object visible for the TAGLIB
        exposeScreenContext(httpRequest) ;
        
        //--- Try to forward to the Screen Map Target
        trace("Forward to " + sTarget + " ... ");
        boolean bForwardOk = false;
        String sMsgForwardError = "Cannot forward to target '" + sTarget + "' ";
        try
        {
            bForwardOk = WebUtil.forward( sTarget, httpRequest, httpResponse );
        } catch (ServletException e)
        {
            throw new TelosysException(sMsgForwardError + " (ServletException) ", e);
        } catch (IOException e)
        {
            throw new TelosysException(sMsgForwardError + " (IOException) ", e);
        }
        if (!bForwardOk)
        {
            throw new TelosysException(sMsgForwardError + " (invalid target) ");
        }
    }
}