package org.objectweb.telosys.common;

import java.io.InputStream;
import java.util.Properties;

/**
 * Trace configuration holder.<br>
 * Keep the "trace" flag by class name.
 * 
 * @author Laurent GUERIN
 *  
 */

public class TraceConfig
{
    private static final String CLASS_NAME = TraceConfig.class.getName() ;

    private static Properties   _flags     = null;  // Flags list (access by class name)

    private static int          _iState    = 0;     // Current State of the configuration

    //-----------------------------------------------------------------------------
    // Init
    //-----------------------------------------------------------------------------
    /**
     * Init from properties
     * @param prop
     */
    public static void init(Properties prop)
    {
        Telosys.info(CLASS_NAME, "init(Properties)");
        _flags = prop;
        _iState++;
        Telosys.info(CLASS_NAME, "init : " + _flags.size() + " classes in the configuration ");
    }

    //-----------------------------------------------------------------------------
    /**
     * Init from properties file name
     * @param sFileName
     */
    public static void init(String sFileName)
    {
        Telosys.info(CLASS_NAME, "init('" + sFileName + "')");
        _flags = Tool.loadProperties(sFileName);
        _iState++;
        Telosys.info(CLASS_NAME, "init : " + _flags.size() + " classes in the configuration ");
    }

    //-----------------------------------------------------------------------------
    /**
     * Init from InputStream
     * @param is
     */
    public static void init(InputStream is)
    {
        Telosys.info(CLASS_NAME, "init(InputStream)");
        _flags = Tool.loadProperties(is);
        _iState++;
        Telosys.info(CLASS_NAME, "init : " + _flags.size() + " classes in the configuration ");
    }


    //-----------------------------------------------------------------------------
    /**
     * Return the current state (numeric stamp) 
     * @return
     */
    public static int getCurrentState()
    {
        return _iState;
    }

    //-----------------------------------------------------------------------------
    /**
     * Return the "trace" flag state for the given class
     * @param sClassName
     * @return
     */
    public static boolean flagIsOn(String sClassName)
    {
        return getFlag(sClassName);
    }

    //-----------------------------------------------------------------------------
    /**
     * Return the current "trace" flag for the given class
     * @param sClassName
     * @return
     */
    public static boolean getFlag(String sClassName)
    {
        if (_flags == null)
        {
            return false;
        }
        else
        {
            String sFlag = _flags.getProperty(sClassName, "0"); // False =
            // default value
            if (sFlag != null)
            {
                if (sFlag.startsWith("1"))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * Set the "trace" flag for the given class 
     * @param sClassName
     * @param bFlag
     */
    public static void setFlag(String sClassName, boolean bFlag)
    {
        if (_flags == null)
        {
            _flags = new Properties();
        }
        if (bFlag == true)
        {
            _flags.setProperty(sClassName, "1");
        }
        else
        {
            _flags.setProperty(sClassName, "0");
        }
        _iState++;
    }

}