package org.objectweb.telosys.common.data;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Iterator implementation designed to iterate on a DataSet (row by row)
 * 
 * @author Laurent GUERIN
 *  
 */
public class DataSetIterator implements Iterator
{
    private DataSet _dataset          = null;

    private int     _iCurrentRow = 0;

    /**
     * Private constructor
     */
    private DataSetIterator() // No default constructor
    {
    }
    
    /**
     * Constructs an Iterator for the given DataSet
     * 
     * @param dataset
     */
    public DataSetIterator(DataSet dataset)
    {
        _dataset = dataset ;
        _iCurrentRow = 0; // before the first row
    }

    /* (non-Javadoc)
     * @see java.util.Iterator#hasNext()
     */
    public boolean hasNext()
    {
        if ( _iCurrentRow < _dataset.getRowCount() )
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /* (non-Javadoc)
     * @see java.util.Iterator#next()
     */
    public Object next()
    {
        if ( _iCurrentRow < _dataset.getRowCount() )
        {
            _iCurrentRow++ ;
            return _dataset.getDataRow(_iCurrentRow) ;            
        }
        else
        {
            throw new NoSuchElementException("No such DataRow in the DataSet");
        }
    }

    /* (non-Javadoc)
     * @see java.util.Iterator#remove()
     */
    public void remove()
    {
        throw new UnsupportedOperationException("No remove operation for DataSet object");
    }
}