package org.objectweb.telosys.common.vo;

import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * Special VOList implementation with loosely typed methods <br>
 * Useful for internal framework usage.
 * 
 * @author Laurent GUERIN
 * @since 1.0.0
 *
 */
public class LooselyTypedVOList extends GenericVOList
{
    //-----------------------------------------------------------------------------
    // CONSTRUCTOR
    //-----------------------------------------------------------------------------
    /**
     * Constructs a new instance
     * @param cl
     */
    public LooselyTypedVOList( Class cl )
    {
        super(cl); // null class checked in super class
    }
    
    //-----------------------------------------------------------------------------
    // V.O. LIST METHODS 
    //-----------------------------------------------------------------------------
    /**
     * Create a new instance of the element type and add it in the list
     * @return
     */
    public Object add()
    {
        //--- Create the list element using the list element type
        Object bean = null ;
        Class cl = super.getElementType();
        if ( cl == null )
        {
            throw new TelosysRuntimeException("Cannot get element type from super class." );
        }
        try
        {
            bean = cl.newInstance();
        } catch (InstantiationException e)
        {
            throw new TelosysRuntimeException("Cannot create bean " + cl.getName(), e);
        } catch (IllegalAccessException e)
        {
            throw new TelosysRuntimeException("Cannot create bean " + cl.getName(), e);
        }

        //--- Add it in the list 
        super.addElement(bean);
        return bean ;
    }
    //-----------------------------------------------------------------------------------------------  
    /**
     * Adds the given element in the list
     * @param vo
     */
    public void add(Object vo)
    {
        super.addElement(vo);
    }
    //-----------------------------------------------------------------------------------------------  
    /**
     * Inserts the given element in the list
     * @param index
     * @param vo
     */
    public void insert(int index, Object vo)
    {
        super.insertElement(index,vo);
    }
    //-----------------------------------------------------------------------------------------------  
    /**
     * Replaces the given element in the list
     * @param index
     * @param vo
     * @return
     */
    public Object replace(int index, Object vo)
    {
        return super.replaceElement(index,vo);
    }
    //-----------------------------------------------------------------------------------------------  
    /**
     * Removes the element located at the given index 
     * @param index
     * @return
     */
    public Object remove(int index)
    {
        return super.removeElement(index);
    }
    //-----------------------------------------------------------------------------------------------  
    /**
     * Removes the given element
     * @param vo
     * @return
     */
    public boolean remove(Object vo)
    {
        return super.removeElement(vo);
    }
    //-----------------------------------------------------------------------------------------------  
    //-----------------------------------------------------------------------------------------------  
    /**
     * Returns the element located at the given index
     * @param index
     * @return
     */
    public Object get(int index)
    {
        return super.getElement(index);
    }
    //-----------------------------------------------------------------------------------------------  
    /**
     * Returns the first element
     * @return
     */
    public Object getFirst()
    {
        return super.getFirstElement();
    }
    //-----------------------------------------------------------------------------------------------  
    /**
     * Returns the next element
     * @return
     */
    public Object getNext()
    {
        return super.getNextElement();
    }
    //-----------------------------------------------------------------------------------------------  
}

