package org.objectweb.telosys.dal.rest;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.rpl.xml.output.DataOutputError;
import org.objectweb.telosys.rpl.xml.output.GenericXmlOutput;
import org.objectweb.telosys.util.XmlUtil;

/**
 * XML response renderer
 * 
 * @author Olivier Laurendeau, Laurent GUERIN
 * 
 * @since 1.0.0
 */
public class DAORendererXML implements DAORenderer
{
    private static final String DAO_RESULT_TAG_NAME = "dao-result" ;

    //private PrintWriter pw = null ;
    
//    /**
//     * @param pw
//     */
//    public DAORendererXML(PrintWriter pw)
//    {
//        super();
//        this.pw = pw;
//    }
    
    //--------------------------------------------------------------------------------------------------
    //public void printResponse( DAOResponse response )
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.rest.DAORenderer#printResponse(javax.servlet.http.HttpServletResponse, org.objectweb.telosys.dal.rest.DAOResponse)
     */
    public void printResponse( HttpServletResponse httpResponse, DAOResponse response ) throws IOException // v 1.0.5
    {
        httpResponse.setContentType(TelosysConst.HTTP_CONTENT_TYPE_XML_UTF8);
        
        PrintWriter pw = httpResponse.getWriter();
        
        prologXml(pw);
        beginningOfXml ( response, pw);
        genReturn ( response, pw);
        DataOutputError dataError = genData ( response, pw);
        GenericXmlOutput.generateOutputErrors(dataError, null, pw); // v 1.0.5
        endOfXml (pw);
        //pw.flush();
        pw.close();
    }

    //--------------------------------------------------------------------------------------------------
    //public void printError( Throwable t )
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.rest.DAORenderer#printError(javax.servlet.http.HttpServletResponse, java.lang.Throwable)
     */
    public void printError( HttpServletResponse httpResponse, Throwable t ) throws IOException // v 1.0.5
	{
        httpResponse.setContentType(TelosysConst.HTTP_CONTENT_TYPE_XML_UTF8);
        
        PrintWriter pw = httpResponse.getWriter();
        
        prologXml(pw);
        beginningOfXml (pw);
        genReturn ( t, pw);
        endOfXml (pw);
        //pw.flush();  
        pw.close();
	}

    //--------------------------------------------------------------------------------------------------
	private void prologXml ( PrintWriter out)
	{
		// out.println("<?xml  version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
		out.println("<?xml version=\"1.0\" encoding=\"" + TelosysConst.XML_PROLOG_ENCODING_UTF8 + "\" ?>"); // v 1.0.5
	}
    //--------------------------------------------------------------------------------------------------
	private void beginningOfXml ( PrintWriter out)
	{
		out.print("<response>" );
	}
    //--------------------------------------------------------------------------------------------------
	private void beginningOfXml ( DAOResponse response, PrintWriter out)
	{
		out.print("<response"
			+ " dao=\""    + response.getDAOClassName() + "\""
			+ " action=\"" + response.getAction() + "\""
			+ " >");
	}
	
	//--------------------------------------------------------------------------------------------------
	// <return> ... </return>
	//--------------------------------------------------------------------------------------------------
	private void genReturn ( DAOResponse response, PrintWriter out)
	{
	    //log.trace ( "genReturn()" );
		out.println("<return"
		        + " code=\"" + response.getRetCode() + "\"" 
		        + " >" );		            
		//--- Flag "FOUND" or "NOT-FOUND" (or nothing)
		if ( response.isNotFound() == true )
		{
			out.println(" <not-found/>" );
		}
		if ( response.isFound() == true )
		{
			out.println(" <found/>" );
		}
				
		/**
		//--- List of exceptions 
		int iCount = response.getNumberOfExceptions() ;
		if ( iCount > 0 )
		{
		    Exception ex = null ;
			out.println("<exceptions>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        ex = response.getException(i);
		        if ( ex != null )
		        {
					out.println(" <exception num=\"" + (i+1) +"\" class=\"" + ex.getClass().getName() + "\" message=\"" + XmlUtil.xmlString( ex.getMessage() ) + "\" />" );		            
		        }
		    }
			out.println("</exceptions>" );
		}
		**/
		
		out.println("</return>" );
		
	}
	//--------------------------------------------------------------------------------------------------
	/**
	 * Return tag for EXCEPTION
	 * @param t
	 * @param out
	 */
	private void genReturn ( Throwable t, PrintWriter out)
	{
	    //log.trace ( "genReturn()" );
		out.println("<return"
		        + " code=\"-1\"" 
		        + " >" );		            
				
		out.println("<exceptions>" );
		Throwable ex = t ;
		int i = 0 ;
		while ( ex != null )
		{
		    i++;
		    genException ( i, ex, out);
		    ex = ex.getCause();
		}		
		out.println("</exceptions>" );
		
		out.println("</return>" );
		
	}
	//--------------------------------------------------------------------------------------------------
	private void genException ( int i, Throwable t, PrintWriter out)
	{
        if ( t != null )
        {
			out.println(" <exception"
			        + " num=\"" + i + "\""
			        + " class=\"" + t.getClass().getName() + "\""
			        + " message=\"" + XmlUtil.xmlString( t.getMessage() ) + "\""
			        + " />" );		            
        }
	}
	
	//--------------------------------------------------------------------------------------------------
	private DataOutputError genData ( DAOResponse response, PrintWriter out) //throws TelosysException
	{
//	    log.trace ( "genData()" );
//		if ( checkPrintWriter(out) == false ) return ;
		
	    DataOutputError dataError = null ;
	    Object oResult = response.getResultObject() ;
	    if ( oResult != null )
	    {
			out.println("<data>" );

// removed in v 1.0.5
//			if ( oResult instanceof GenericVOList )
//			{
//                GenericVOList volist = (GenericVOList) oResult ;
//                GenericVOListWriter.toXml(out, volist, "list", "item" );
//			}
//			else
//			{
//			    BeanWriter.toXml(out, oResult, "entity");
//			}
			
//			// Ex : "Employee" for "org.demo.Employee"
//			String sShortClassName = ClassUtil.getSimpleName ( oResult.getClass() ); //  v 1.0.5
//			GenericXmlOutput.generateDataElement(oResult, sShortClassName, out); //  v 1.0.5
			
			dataError = GenericXmlOutput.generateDataElement(oResult, DAO_RESULT_TAG_NAME, out); //  v 1.0.5
			
			
			out.println("</data>" );         
	    }
	    else
	    {
			out.println("<data void=\"true\" />" );
	    }
		return dataError ;
	}

	//--------------------------------------------------------------------------------------------------
	private void endOfXml ( PrintWriter out ) 
	{
	    //log.trace ( "endOfXml()" );
		//if ( checkPrintWriter(out) == false ) return ;
		out.println("</response>");
	}    
}
