package org.objectweb.telosys.dal.sql;

import java.math.BigDecimal;

import org.objectweb.telosys.common.TelosysException;

/**
 * Utility class
 * 
 * @author Laurent GUERIN
 *  
 */

public class Cast
{

    /**
     * Transforms the given string value to its corresponding value in the expected type
     * @param s
     * @param iType ParamType.X constants
     * @return
     * @throws TelosysException
     */
    public static Object toObject(String s, int iType) throws TelosysException
    {
        if ( s == null )
        {
            return null ;
        }
        
        try 
        {
            switch (iType)
            {
            case ParamType.BOOLEAN :
                return Boolean.valueOf(s); // "true" (ignore case) / other = false

            case ParamType.BYTE :
                return Byte.valueOf(s); // throws NumberFormatException
            
            case ParamType.DOUBLE :
                return Double.valueOf(s); // throws NumberFormatException
            
            case ParamType.FLOAT :
                return Float.valueOf(s); // throws NumberFormatException
            
            case ParamType.INTEGER :
                return Integer.valueOf(s); // throws NumberFormatException
            
            case ParamType.LONG :
                return Long.valueOf(s); // throws NumberFormatException
            
            case ParamType.SHORT :
                return Short.valueOf(s); // throws NumberFormatException
            
            case ParamType.STRING :
                return s;

            case ParamType.DATE : 
                return java.sql.Date.valueOf(s); // "yyyy-mm-dd" ( throws IllegalArgumentException )

            case ParamType.TIMESTAMP :
                return java.sql.Timestamp.valueOf(s); // "yyyy-mm-dd hh:mm:ss.ffffff" ( throws IllegalArgumentException )

            case ParamType.TIME : // v 1.1.0
                return java.sql.Time.valueOf(s); // "hh:mm:ss" ( throws IllegalArgumentException )

            case ParamType.BIGDECIMAL : // v 1.1.0
                return new BigDecimal(s); // throws NumberFormatException

            default :
                throw new TelosysException("Cast.toObject() : Unknowned type " + iType);
            }
        } catch (NumberFormatException e)
        {
            throw new TelosysException("Cast.toObject() : Invalid number '" + s + "' ", e);
        } catch (IllegalArgumentException e)
        {
            throw new TelosysException("Cast.toObject() : Invalid date (ISO) '" + s + "' ", e);
        }
    }
}