package org.objectweb.telosys.dal.sql;

import java.util.*;
import java.sql.*;

/**
 * 
 * Factory designed to create standalone connections <br>
 * ( connections that doesn't belong to a pool )
 * 
 * This class is a "ConnectionProvider"
 * 
 * @author Laurent GUERIN
 *  
 */

public class SqlConnectionFactory extends SqlConnectionProvider
{
    //-----------------------------------------------------------------------------
    /**
     * Constructor
     * @param sName the connection provider name
     * @param sDriver the JDBC driver to use
     * @param sUrl the JDBC URL 
     * @param prop the connection properties
     * @param iIsolationLevel the "isolation level"
     */
    public SqlConnectionFactory(String sName, String sDriver, String sUrl, Properties prop, int iIsolationLevel)
    {
        super(sName, sDriver, sUrl, prop, iIsolationLevel);
        trace("Constructor...");
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.sql.SqlConnectionProvider#getConnection()
     */
    public synchronized Connection getConnection() throws Exception
    {
        trace("getConnection()");
        Connection stdCon = createStandardJdbcConnection();
        return new SqlConnection(stdCon);
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.sql.SqlConnectionProvider#isPool()
     */
    public boolean isPool()
    {
        return false; // No this connection provider is not a pool
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.sql.ConnectionProvider#getType()
     */
    public int getType()
    {
        return ConnectionProvider.FACTORY ;  // This connection provider is a "FACTORY"
    }

}