package org.objectweb.telosys.rpl;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysServlet;
import org.objectweb.telosys.rpl.xml.output.ScreenActionXmlOutput;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.uil.RequestEnv;

/**
 * Front servlet for ACTIONS requests <br>
 * Can be called only with "POST" http method
 * 
 * @author Laurent GUERIN
 *  
 */
public class ScreenRequestServlet extends TelosysServlet // HttpServlet
{
    //private final static String RESPONSE_CONTENT_TYPE = "text/xml;charset=utf-8" ; // suppr in v 1.0.5
    
    private final static String INPUT_CHARSET = "utf-8" ; // "UTF-8", "ISO-8859-1", etc...
    
    private boolean bSynchronizeActionOnScreenSession = true ;
    
    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.GenericServlet#init()
     */
    public void init() throws ServletException
    {
        trace("init()");
    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void doGet(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        trace("========================================");
        trace("doGet()...");
        //--- The response is always XML
        //httpResponse.setContentType(RESPONSE_CONTENT_TYPE); // v 1.0.5
        httpResponse.setContentType(TelosysConst.HTTP_CONTENT_TYPE_XML_UTF8); // v 1.0.5
        //--- "GET" is not processed ( only "POST" )
        sendErrorResponse(httpRequest, httpResponse, ScreenResponse.INVALID_HTTP_METHOD, "HTTP GET method is not supported ! ", null);
    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void doPost(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        trace("========================================");
        trace("doPost()");
        //--- The response is always XML
        //httpResponse.setContentType(RESPONSE_CONTENT_TYPE); // v 1.0.5
        httpResponse.setContentType(TelosysConst.HTTP_CONTENT_TYPE_XML_UTF8); // v 1.0.5
        //--- Process the input request
        process(httpRequest, httpResponse);
    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.Servlet#destroy()
     */
    public void destroy()
    {
        trace("destroy()");
    }

    //---------------------------------------------------------------------------
    /**
     * Process the http request <br>
     * . get the current ScreenSession <br>
     * . build the ScreenRequest by parsing the XML input stream <br>
     * . get the ScreenContext <br>
     * . call the action processor to execute the action <br>
     * . generate the response <br>
     * 
     * @param httpRequest
     * @param httpResponse
     * @throws ServletException
     * @throws IOException
     */
    private void process(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        int iErrorCode = 0 ;
        String sErrorMessage = null ;
        
        //--- Servlet output stream
        PrintWriter output = httpResponse.getWriter();
        if (output == null)
        {
            // Cannot build Http Response Error => just log
            error("Output stream is null ");
            return;
        }

        try
        {
            //--- Get the current "Screen Session" ( create a "dummy user" session if Authentication Not Requiered )
            trace(" --> Get ScreenSession ...");
            iErrorCode = -5 ;
            sErrorMessage = "Cannot get screen session object" ;
            
            //ScreenSession screenSession = ScreenRequestProcessor.getScreenSession(httpRequest); // v 1.1.0
            ScreenSession screenSession = ScreenSessionManager.getScreenSessionOrThrowException(httpRequest); // v 1.1.0

            //--- Check Servlet input stream
            iErrorCode = -2 ;
            sErrorMessage = "No input stream" ;
            InputStream input = httpRequest.getInputStream();
            if (input == null)
            {
                throw new TelosysException("Input stream is null");
            }
            
            //--- Create a ScreenRequest from the XML input stream ( XML parsing )
            trace(" --> Parse XML input stream ...");
            iErrorCode = -3 ;
            sErrorMessage = "Request parsing error" ;
//            ScreenRequest screenRequest = ScreenRequestProcessor.parseInputStream(input, INPUT_CHARSET);
            ScreenRequest screenRequest = new ScreenRequest(httpRequest);
            ScreenRequestProcessor.parseInputStream(screenRequest, input, INPUT_CHARSET);

            //--- Get a ScreenResponse
            trace(" --> Get ScreenResponse ...");
            iErrorCode = -4 ;
            sErrorMessage = "Cannot get screen response object" ;
            //ScreenResponse screenResponse = ScreenRequestProcessor.getScreenResponse(screenRequest);
            ScreenResponse screenResponse = ScreenRequestProcessor.getScreenResponse(screenRequest, httpResponse); // v 0.9.9

            //--- Get the ScreenContext associated with the request Screen ID (can be null if not yet opened and action = "open")
            trace(" --> Get ScreenContext ...");
            iErrorCode = -6 ;
            sErrorMessage = "Cannot get screen context object" ;
            ScreenContext screenContext = ScreenRequestProcessor.getScreenContext(screenRequest, screenSession);

            //--- Execute the action
            iErrorCode = -7 ;
            sErrorMessage = "Error in action execution ( action = '" + screenRequest.getAction() + "' ) ";
            if ( bSynchronizeActionOnScreenSession )
            {
                //--- Execute the action in synchronized mode to avoid concurrency problems
                trace(" --> Execute ACTION ( synchronized ) ...");
                ScreenRequestProcessor.executeSynchronizedAction(screenSession, screenContext, screenRequest, screenResponse);
            }
            else
            {
                //--- Execute the action without synchronization
                trace(" --> Execute ACTION ( not synchronized ) ...");
                ScreenRequestProcessor.executeAction(screenSession, screenContext, screenRequest, screenResponse);                                
            }
            
//            if ( screenRequest.hasViewsAttribute() )
//            {
//                //--- Define renderers in the response 
//                //screenResponse.useViews( screenRequest.getViews() );
//            }

            //--- EXPOSE THE SCREEN CONTEXT for TAGLIB and EXPOSE DATA ELEMENTS for standard JSP E.L. 
            //--- Create a request environment and and expose it at "request level" 
            //RequestEnv.setEnv(httpRequest, screenSession, screenContext);
            //RequestEnv.setEnv(httpRequest, screenContext); // v 0.9.9
            RequestEnv.setEnv(httpRequest, screenContext, RequestEnv.EXPOSE_DATA ); // v 1.0.1

//            //--- Expose Screen Data Elements at "request level" 
//            if ( screenContext != null ) // can be null if action = "open"
//            {
//	            ScreenData data = screenContext.getCurrentData();
//	            if ( data != null )
//	            {
//	                data.exposeData(httpRequest); // since v 1.0.1 : Done in "RequestEnv.setENV"
//	            }                
//            }
            
            //--- OUTPUT : Generate the XML response
            iErrorCode = -8 ;
            sErrorMessage = "Error in output generation" ;
            trace(" --> Generate output ...");
            ScreenActionXmlOutput.generate(screenRequest, screenResponse, output, httpRequest, httpResponse );

            output.close();

//        } catch (TelosysException ex)
//        {
//            //sendErrorResponse(httpResponse, 1003, "TelosysException : " + ex.getMessage());
//            error("TelosysException : " + ex.getMessage());
//            sendErrorResponse(httpResponse, iErrorCode, sErrorMessage, ex);
//            ex.printStackTrace();
//        } catch (TelosysRuntimeException ex)
//        {
//            error("TelosysRuntimeException : " + ex.getMessage());
//            //sendErrorResponse(httpResponse, 1004, "TelosysRuntimeException : " + ex.getMessage());
//            sendErrorResponse(httpResponse, iErrorCode, sErrorMessage, ex);
//            ex.printStackTrace();
//        } catch (Throwable t)
//        {            
//            error("Throwable : " + t.getMessage() );
//            //sendErrorResponse(httpResponse, 1005, "Throwable Exception : " + t.getMessage());
//            sendErrorResponse(httpResponse, iErrorCode, sErrorMessage, t);
//            t.printStackTrace();
//        }
	    } catch (Throwable t)
	    {            
	        //processError( httpResponse, iErrorCode, sErrorMessage, t);
	        processError(httpRequest, httpResponse, iErrorCode, sErrorMessage, t); // v 0.9.9
	    }
    }
    //---------------------------------------------------------------------------
    /**
     * Error processing 
     * 
     * @param httpRequest
     * @param httpResponse
     * @param iReturnCode
     * @param sErrorMessage
     * @param t
     */
    private void processError(HttpServletRequest httpRequest, HttpServletResponse httpResponse, int iReturnCode, String sErrorMessage, Throwable t) 
    {
        error(" === ACTION PROCESSING ERROR : " + iReturnCode + " : "+ sErrorMessage );
        if ( t != null )
        {
            error(" === EXCEPTION : '" + t.getClass() + "' : " + t.getMessage() );
            Throwable cause = t.getCause() ;
            if ( cause != null  )
            {
                error(" === CAUSE : '" + cause.getClass() + "' : " + cause.getMessage() );
            }
            t.printStackTrace();
        }
        //error(" === SENDING ERROR RESPONSE : " + iReturnCode + " : " + sErrorMessage );
        sendErrorResponse(httpRequest, httpResponse, iReturnCode, sErrorMessage, t);
    }
    
    //---------------------------------------------------------------------------
    /**
     * Generates an error response 
     * 
     * @param httpRequest
     * @param response
     * @param iReturnCode
     * @param sMsg
     * @param t
     */
    private void sendErrorResponse(HttpServletRequest httpRequest, HttpServletResponse response, int iReturnCode, String sMsg, Throwable t) 
    {
        //--- Create a ScreenResponse object 
        //ScreenResponse screenResponse = new ScreenResponse("", "", "", "");
        //ScreenResponse screenResponse = new ScreenResponse(response, "", "", "", ""); // v 0.9.9
        ScreenResponse screenResponse = new ScreenResponse(httpRequest, response, "", "", "", ""); // v 0.9.9 bis
        
        //--- Set the return code and message 
        // screenResponse.setReturnCode(iReturnCode);// v 1.0.2
        // screenResponse.setReturnMessage(sMsg); // v 1.0.2
        screenResponse.setActionReturn(iReturnCode, sMsg); // v 1.0.2
        
        if ( t != null )
        {
            //screenResponse.addError(t); // v 1.0.5
            screenResponse.addException(t); // v 1.0.5
            if ( t.getCause() != null )
            {
                //screenResponse.addError(t.getCause());  // v 1.0.5       
                screenResponse.addException(t.getCause()); // v 1.0.5
            }
        }
        
        //--- Send the HTTP Response
        try
        {
            //trace("error - step 1");
            PrintWriter out = response.getWriter();
            if (out == null)
            {
                error("Servlet output stream is null ");
                return;
            }
            //trace("error - step 1");
            //--- Generate the XML response
            ScreenActionXmlOutput.generateError(screenResponse, out);
            
            //trace("error - step 3");
            out.close();
            //trace("error - step 4");

        } catch (Exception e)
        {
            //--- Log the error
            error("Error : " + e.getMessage());
        }
        //trace("error - end ");
    }
    //---------------------------------------------------------------------------
}