package org.objectweb.telosys.rpl;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.screen.core.ScreenData;
import org.objectweb.telosys.screen.core.ScreenDataElements;

/**
 * The response object populated by a "Screen Action" <br>
 * 
 * @author Laurent GUERIN
 * 
 */
public class ScreenResponse extends GenericResponse // TelosysObject // extends GenericResponse v 1.0.5
{
    public final static int ACTION_OK       = 0;

    public final static int ACTION_CANCELED = 1;

    public final static int ACTION_ERROR    = 2;

    public final static int INVALID_ACTION  = 3 ;
    
    public final static int INVALID_HTTP_METHOD  = 4 ;
    
    //public final static int INVALID_SCREEN_ID    = 101 ; // v 1.0.2

    //public final static int SCREEN_NOT_FOUND     = 102 ; // v 1.0.2

//    //-----------------------------------------------------------------------------------------------
//    // Current http response object 
//    //-----------------------------------------------------------------------------------------------
//
//    private HttpServletRequest  _httpRequest  = null ; // since v 0.9.9 // removed in v 1.0.5 (see GenericResponse)
//    
//    private HttpServletResponse _httpResponse = null ; // since v 0.9.9 // removed in v 1.0.5 (see GenericResponse)
    
    //-----------------------------------------------------------------------------------------------
    // Screen Request parameters
    //-----------------------------------------------------------------------------------------------

    private String          _sScreenName    = null;

    private String          _sScreenId      = null;

    private String          _sAction        = null;

    private String          _sProcName      = null;

    //-----------------------------------------------------------------------------------------------
    // Screen Response data
    //-----------------------------------------------------------------------------------------------
    ScreenDataElements      _dataElements   = null ; // Data elements to return in the response
    
//    private Hashtable       _htValues       = null; // Values to return in the response // removed in v 1.0.5 (see GenericResponse)
//
//    private LinkedList      _listExceptions = null; // List of all the exceptions // removed in v 1.0.5 (see GenericResponse)
//
//    private LinkedList      _listErrors     = null; // List of all the errors // removed in v 1.0.5 (see GenericResponse)
//
//    private LinkedList      _listMessages   = null; // List of all the app. messages // removed in v 1.0.5 (see GenericResponse)
//
//    private int             _iReturnCode    = ACTION_OK ;    // default = "0" ( = OK ) // removed in v 1.0.5 (see GenericResponse)
//
//    private String          _sReturnMessage = null ; // default = "Ok" // removed in v 1.0.5 (see GenericResponse)

    //private boolean _bNotFound = false;

    private boolean         _bFlagNotFound  = false;

    private boolean         _bFlagFound     = false;
    
//    private String          _sRedirection   = null ; // v 0.9.9 // removed in v 1.0.5 (see GenericResponse)

    //-----------------------------------------------------------------------------------------------
    // constructor(s)
    //-----------------------------------------------------------------------------------------------

// Removed in v 0.9.9
//    public ScreenResponse(String sScreenName, String sScreenId, String sAction, String sProcName)
//    {
//        _sScreenName = sScreenName;
//        _sScreenId = sScreenId;
//        _sAction = sAction;
//        _sProcName = sProcName;
//    }

    // New constructor since v 0.9.9
    /**
     * Constructor
     * @param httpRequest
     * @param httpResponse
     * @param sScreenName
     * @param sScreenId
     * @param sAction
     * @param sProcName
     */
    public ScreenResponse(HttpServletRequest httpRequest, HttpServletResponse httpResponse, 
            String sScreenName, String sScreenId, String sAction, String sProcName)
    {
        super(httpRequest, httpResponse); // added in v 1.0.5 (see GenericResponse)
//        _httpRequest = httpRequest ; // removed in v 1.0.5 (see GenericResponse)
//        _httpResponse = httpResponse; // removed in v 1.0.5 (see GenericResponse)
        
        _sScreenName = sScreenName;
        _sScreenId   = sScreenId;
        _sAction     = sAction;
        _sProcName   = sProcName;
    }

    //-----------------------------------------------------------------------------------------------
    // methods
    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the Screen Name that will be returned in the response
     * @return
     */
    public String getScreenName()
    {
        return _sScreenName;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the Screen ID that will be returned in the response
     * @return
     */
    public String getScreenId()
    {
        return _sScreenId;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the Screen Action that will be returned in the response
     * @return
     */
    public String getAction()
    {
        return _sAction;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the Procedure Name (if any) that will be returned in the response
     * @return
     */
    public String getProcName()
    {
        return _sProcName;
    }


    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the action predefined return code <br>
     * NB : reserved for the framework usage ( do not use in the application level )
     * @param iReturnCode
     */
    public void setActionReturn(int iReturnCode)
    {
        switch (iReturnCode)
        {
        case ACTION_OK :
//            _iReturnCode = ACTION_OK;
//            _sReturnMessage = "OK";
            setActionReturn(iReturnCode, "OK");
            break;
        case ACTION_CANCELED :
//            _iReturnCode = ACTION_CANCELED;
//            _sReturnMessage = "Action canceled";
            setActionReturn(iReturnCode, "Action canceled");
            break;
        case ACTION_ERROR :
//            _iReturnCode = ACTION_ERROR;
//            _sReturnMessage = "Action error (abnormal termination)";
            setActionReturn(iReturnCode, "Action error (abnormal termination)");
            break;
        default :
//            _iReturnCode = 99999;
//            _sReturnMessage = "Unknown action return code " + iReturnCode;
            setActionReturn(iReturnCode, "");
            break;
        }
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set the action predefined return code <br>
     * NB : reserved for the framework usage ( do not use in the application level )
     * @param iReturnCode
     * @param sReturnMsg
     */
    //public void setActionReturn(int iReturnCode, String sReturnMsg )
    /* package */ void setActionReturn(int iReturnCode, String sReturnMsg ) // v 1.0.5
    {
        if ( iReturnCode == ACTION_OK )
        {
            //if ( _iReturnCode != ACTION_OK )  // v 1.0.5
            if ( getReturnCode() != ACTION_OK )  // v 1.0.5
            {
                return ; // An error code has been set before => do not change it
            }
            else 
            {
                //if ( _sReturnMessage != null )  // v 1.0.5
                if ( getReturnMessage() != null )  // v 1.0.5
                {
                    return ; // Code is OK, but the message has been set => do not change it 
                }
            }
        }
        
        switch (iReturnCode)
        {
        case ACTION_OK :
        case ACTION_CANCELED :
        case ACTION_ERROR :
        case INVALID_ACTION :
            //_iReturnCode    = iReturnCode ; // v 1.0.5
            //_sReturnMessage = sReturnMsg ; // v 1.0.5
            setReturnCode(iReturnCode); // v 1.0.5
            setReturnMessage(sReturnMsg); // v 1.0.5
            break;
            
        default : // Unknown action code 
            //_iReturnCode = 99999; // v 1.0.5
            //_sReturnMessage = "Unknown action return code " + iReturnCode ; // v 1.0.5
            setReturnCode(99999); // v 1.0.5
            setReturnMessage("Unknown action return code " + iReturnCode ); // v 1.0.5
            break;
        }
    }
    
//    //-----------------------------------------------------------------------------------------------
//    /**
//     * Set the action return code <br>
//     * This method is dedicated to the application usage ( typically in the Screen Manager )<br>
//     * NB : The return code set by the application can be overwritten by the framework if an error occurs
//     * @param code
//     */
//    public void setReturnCode(int code)
//    {
//        _iReturnCode = code;
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public int getReturnCode()
//    {
//        return _iReturnCode;
//    }
    
//    //-----------------------------------------------------------------------------------------------
//    /**
//     * Set the action return message <br>
//     * This method is dedicated to the application usage ( typically in the Screen Manager )
//     * NB : The return message set by the application can be overwritten by the framework if an error occurs
//     * @param message
//     */
//    public void setReturnMessage(String message)
//    {
//        _sReturnMessage = message;
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public String getReturnMessage()
//    {
//        return _sReturnMessage != null ? _sReturnMessage : "OK" ; // v 1.0.2 ( "OK" by default )
//    }

    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    //    public void setFlagNotFound(boolean bFlag)
    //    {
    //        _bNotFound = bFlag;
    //    }

    //-----------------------------------------------------------------------------------------------
    // Flag : return a tag "not found" or no
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the "Not Found" flag value
     * @return
     */
    public boolean isFlagNotFound()
    {
        return _bFlagNotFound;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the "Not Found" flag to "TRUE", and the "Found" flag to false
     */
    public void setFlagNotFound()
    {
        _bFlagNotFound = true;
        _bFlagFound = false;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the "Not Found" flag to "FALSE"
     */
    public void unsetFlagNotFound()
    {
        _bFlagNotFound = false;
    }

    //-----------------------------------------------------------------------------------------------
    // Flag : return a tag "found" or no
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the "Found" flag value
     * @return
     */
    public boolean isFlagFound()
    {
        return _bFlagFound;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the "Found" flag to "TRUE", and the "NotFound" flag to "FALSE"
     */
    public void setFlagFound()
    {
        _bFlagFound = true;
        _bFlagNotFound = false;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the "Found" flag to "FALSE"
     */
    public void unsetFlagFound()
    {
        _bFlagFound = false;
    }

//    //-----------------------------------------------------------------------------------------------
//    // Messages
//    //-----------------------------------------------------------------------------------------------
//    public void addMessage(ScreenMessage message)
//    {
//        if (_listMessages == null)
//        {
//            _listMessages = new LinkedList();
//        }
//
//        if (message != null)
//        {
//            _listMessages.add(message);
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public int getNumberOfMessages()
//    {
//        if (_listMessages == null)
//        {
//            return 0;
//        }
//        else
//        {
//            return _listMessages.size();
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public ScreenMessage getMessage(int index)
//    {
//        if (_listMessages == null)
//        {
//            return null;
//        }
//        else
//        {
//            Object obj = _listMessages.get(index);
//            if (obj instanceof ScreenMessage)
//            {
//                return (ScreenMessage) obj;
//            }
//            else
//            {
//                error("getMessage(" + index + ") : not a ScreenMessage instance ");
//                return null;
//            }
//        }
//    }

//    //-----------------------------------------------------------------------------------------------
//    // Errors
//    //-----------------------------------------------------------------------------------------------
//    public void addError(ScreenError error)
//    {
//        if (_listErrors == null)
//        {
//            _listErrors = new LinkedList();
//        }
//
//        if (error != null)
//        {
//            _listErrors.add(error);
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public int getNumberOfErrors()
//    {
//        if (_listErrors == null)
//        {
//            return 0;
//        }
//        else
//        {
//            return _listErrors.size();
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public ScreenError getError(int index)
//    {
//        if (_listErrors == null)
//        {
//            return null;
//        }
//        else
//        {
//            Object obj = _listErrors.get(index);
//            if (obj instanceof ScreenError)
//            {
//                return (ScreenError) obj;
//            }
//            else
//            {
//                error("getError(" + index + ") : not a ScreenError instance ");
//                return null;
//            }
//        }
//    }

    //-----------------------------------------------------------------------------------------------
    // Exceptions
    //-----------------------------------------------------------------------------------------------
//    public void addError(Exception ex)
//    {
//        if (_listExceptions == null)
//        {
//            _listExceptions = new LinkedList();
//        }
//
//        if (ex != null)
//        {
//            _listExceptions.add(ex);
//        }
//    }
//
//	/**
//	 * @param th
//	 * @deprecated use addException 
//	 */
//	public void addError(Throwable th) 
//	{
//	    addException(th);
//	}
//    public void addError(Throwable th)
//    {
//        if (_listExceptions == null)
//        {
//            _listExceptions = new LinkedList();
//        }
//
//        if (th != null)
//        {
//            _listExceptions.add(th);
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public int getNumberOfExceptions()
//    {
//        if (_listExceptions == null)
//        {
//            return 0;
//        }
//        else
//        {
//            return _listExceptions.size();
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    //public Exception getException(int index)
//    public Throwable getException(int index)
//    {
//        if (_listExceptions == null)
//        {
//            return null;
//        }
//        else
//        {
//            Object obj = _listExceptions.get(index);
//            if (obj instanceof Throwable)
//            {
//                return (Throwable) obj;
//            }
//            else
//            {
//                error("getException(" + index + ") : not a Throwable instance ");
//                return null;
//            }
//        }
//    }
//
    //-----------------------------------------------------------------------------------------------
    // Data
    //-----------------------------------------------------------------------------------------------
//    public ScreenData getData()
//    {
//        return _Data;
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public void setData(ScreenData data)
//    {
//        _Data = data;
//    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the Data Elements that will be returned in the response
     * @return
     */
    public ScreenDataElements getDataElements()
    {
//        if ( _Data != null )
//        {
//            return _Data.getElements() ;            
//        }
//        return null ;
        return _dataElements ;
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set the Data Elements to return in the response from the given Screen Data
     * @param data
     */
    public void setDataElements(ScreenData data)
    {
      if ( data != null )
      {
          _dataElements = data.getElements() ;            
      }
      else
      {
          _dataElements = null ;                      
      }
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set the Data Elements to return in the response
     * @param dataElements
     */
    public void setDataElements(ScreenDataElements dataElements)
    {
        _dataElements = dataElements;
    }

//    //-----------------------------------------------------------------------------------------------
//    // Values
//    //-----------------------------------------------------------------------------------------------
//    public void setValue(String sName, String sValue)
//    {
//        if (_htValues == null)
//        {
//            _htValues = new Hashtable();
//        }
//
//        if (sName != null)
//        {
//            _htValues.put(sName, sValue);
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public void setValue(String sName, int iValue)
//    {
//        if (_htValues == null)
//        {
//            _htValues = new Hashtable();
//        }
//
//        if (sName != null)
//        {
//            _htValues.put(sName, ""+iValue);
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public String getValue(String sName)
//    {
//        if (_htValues == null)
//        {
//            return null;
//        }
//        else
//        {
//            return (String) _htValues.get(sName);
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public int getNumberOfValues()
//    {
//        if (_htValues == null)
//        {
//            return 0;
//        }
//        else
//        {
//            return _htValues.size();
//        }
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public Enumeration getValueNames()
//    {
//        if (_htValues == null)
//        {
//            return null;
//        }
//        else
//        {
//            return _htValues.keys();
//        }
//    }

//    //-----------------------------------------------------------------------------------------------
//    /**
//     * Force the client-side (the browser) to redirect to the given URL <br>
//     * ( replace the current screen by the given URL )
//     * @param sURL
//     */
//    public void forceRedirect ( String sURL )
//    {
//        if ( sURL != null )
//        {
//            if ( _httpResponse != null )
//            {
//                _sRedirection = sURL ;
//                _httpResponse.setHeader(TelosysConst.TELOSYS_REDIRECT, _sRedirection );
//            }
//            else
//            {
//                throw new TelosysRuntimeException("forceRedirect : HttpResponse is null");
//            }
//        }
//    }
//    
//    //-----------------------------------------------------------------------------------------------
//    /**
//     * Force the client-side (the browser) to redirect to the given generic URI <br>
//     * ( replace the current screen by the given object URL )
//     * @param uri
//     */
//    public void forceRedirect ( GenericURI uri )
//    {
//        if ( uri != null )
//        {
//            forceRedirect( uri.getFullURL( _httpRequest ) );
//        }
//    }
//    
//    //-----------------------------------------------------------------------------------------------
//    /**
//     * Returns the redirection URL set by 'forceRedirect' or null if none
//     * @return
//     */
//    public String getRedirection()
//    {
//        return _sRedirection ;
//    }

    //-----------------------------------------------------------------------------------------------
    //    public void storeDataElement(Object objData)
    //    {
    //        if (_Data != null)
    //        {
    //            _Data.store(objData);
    //        }
    //    }

    //-----------------------------------------------------------------------------------------------
    public String toString()
    {
        //--- New version in v 1.0.4
        
        StringBuffer sb = new StringBuffer(256);
        sb.append("ScreenResponse : " );

        sb.append("\n . "
                + "screen-name='" + _sScreenName + "' " 
                + "screen-id='" + _sScreenId + "' " 
                + "action='" + _sAction + "' " 
                + "proc='" + _sProcName + "' ");
        
        //sb.append("\n . return : code='" + _iReturnCode + "' message='" + _sReturnMessage + "' " ); // v 1.0.5
        sb.append( toString(RETURN) ); // v 1.0.5
        
        sb.append("\n . flags : found='" + _bFlagFound + "' notFound='" + _bFlagNotFound + "' " );
        
        //sb.append("\n . redirection='" + _sRedirection + "'" );
        sb.append( toString(REDIRECTION) ); // v 1.0.5

        sb.append( toString(ERRORS + EXCEPTIONS + MESSAGES + VALUES ) ); // v 1.0.5
//        sb.append("\n --- errors :" );
//        if ( _listErrors != null )
//        {
//	        for ( int i = 0 ; i <_listErrors.size() ; i++ )
//	        {
//	            ScreenError error = (ScreenError) _listErrors.get(i);
//	            sb.append( "\n . " + error );
//	        }
//        }
//        else
//        {
//            sb.append( "\n   (no error)" );
//        }
        
//        sb.append("\n --- exceptions :" );
//        if ( _listExceptions != null )
//        {
//	        for ( int i = 0 ; i <_listExceptions.size() ; i++ )
//	        {
//	            Throwable ex = (Throwable) _listExceptions.get(i);
//	            sb.append( "\n . " + ex );
//	        }
//        }
//        else
//        {
//            sb.append( "\n   (no exception)" );
//        }
        
//        sb.append("\n --- messages :" );
//        if ( _listMessages != null )
//        {
//	        for ( int i = 0 ; i <_listMessages.size() ; i++ )
//	        {
//	            ScreenMessage m = (ScreenMessage) _listMessages.get(i);
//	            sb.append( "\n . " + m );
//	        }
//        }
//        else
//        {
//            sb.append( "\n   (no exception)" );
//        }
        
//        sb.append("\n --- values :" ); 
//        if ( _htValues != null )
//        {
//            Enumeration e = _htValues.keys();
//            while ( e.hasMoreElements() )
//            {
//                Object key = e.nextElement();
//                sb.append( "\n . '" + key + "' : '" +  _htValues.get(key) + "'" );
//            }
//        }
//        else
//        {
//            sb.append( "\n   (no value)" );
//        }
        
        //--- Data
        sb.append("\n --- data elements :" ); 
        if ( _dataElements != null )
        {
            sb.append( "\n" + _dataElements.toString() );
        }
        else
        {
            sb.append( "\n   (no data element)" );                        
        }
        
        return sb.toString() ;
    }
    //-----------------------------------------------------------------------------------------------
}