package org.objectweb.telosys.rpl;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysFactory;
import org.objectweb.telosys.common.TelosysServlet;
import org.objectweb.telosys.common.View;
import org.objectweb.telosys.rpl.xml.output.ServiceXmlOutput;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.service.IServiceProvider;
import org.objectweb.telosys.service.ScreenService;
import org.objectweb.telosys.service.ServiceRequest;
import org.objectweb.telosys.service.ServiceResponse;
import org.objectweb.telosys.service.ServiceResultHolder;
import org.objectweb.telosys.util.web.WebUtil;

/**
 * Front servlet for service request <br>
 * Can be called via "GET" and "POST" http methods
 * 
 * @author Laurent GUERIN
 *  
 */
public class ServiceServlet extends TelosysServlet //extends HttpServlet
{
    private final static String SERVICE_RESPONSE = "TELOSYS_SERVICE_RESPONSE";

    //private final static int    ACTION_INVALID   = -1;

    private final static int    ACTION_NONE      = 0;

    private final static int    ACTION_GET       = 1;

    //---------------------------------------------------------------------------

    private final static int ERROR_CANNOT_GET_SERVICE        = 1 ;
    private final static int ERROR_CANNOT_GET_ACTION         = 2 ;
    private final static int ERROR_INVALID_ACTION            = 3 ;
    private final static int ERROR_CANNOT_GET_SERVICE_ENV    = 4 ;
    private final static int ERROR_CANNOT_GET_SCREEN_SESSION = 5 ;
    private final static int ERROR_IN_SERVICE_EXECUTION      = 6 ;


    private IServiceProvider   _serviceProvider = null;

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.GenericServlet#init()
     */
    public void init() throws ServletException
    {
    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        process(request, response);
    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        process(request, response);
    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.Servlet#destroy()
     */
    public void destroy()
    {
    }

    //---------------------------------------------------------------------------
    /**
     * Request processing
     * 
     * @param httpRequest
     * @param httpResponse
     * @throws ServletException
     * @throws IOException
     */
    private void process(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        String sRenderer = null;
        String sSessionKey = null;
        
//        int iParamCount = 0 ;
//        Enumeration paramNames = httpRequest.getParameterNames();
//        while ( paramNames.hasMoreElements() )
//        {
//            paramNames.nextElement();
//            iParamCount++;
//        }
//        trace("Param count = " + iParamCount);
        

        // Prevent caching 
        WebUtil.noCache(httpResponse);
        
        //------------------------------------------------------------------------
        //--- 1) Get the service
        //------------------------------------------------------------------------
        String sServiceName = "(undefined)";
        ScreenService oScreenService = null ;
        try
        {
            //--- Get the service name
            sServiceName = getServiceNameFromURI(httpRequest);

            if ( null == sServiceName )
            {
                String sMsg = "Cannot get service name from URL"  ;
                error(sMsg);
                ServiceResponse serviceResponse = new ServiceResponse("", httpRequest, httpResponse, null );
                sendXmlErrorResponse(httpResponse, serviceResponse, ERROR_CANNOT_GET_SERVICE, sMsg);
                return ;
            }

            //--- Get a service instance from the provider
            oScreenService = getServiceFromProvider(sServiceName);
            
            if ( null == oScreenService )
            {
                String sMsg = "Cannot get service from provider (" + sServiceName +" )" ;
                error(sMsg);
                ServiceResponse serviceResponse = new ServiceResponse(sServiceName, httpRequest, httpResponse, null );
                sendXmlErrorResponse(httpResponse, serviceResponse, ERROR_CANNOT_GET_SERVICE, sMsg);
                return ;
            }
            
        } catch (Throwable t)
        {
            String sMsg = "Cannot get service (" + sServiceName + ")" ;
            error(sMsg, t);
            ServiceResponse serviceResponse = new ServiceResponse(sServiceName, httpRequest, httpResponse, null );
            serviceResponse.addException(t);                
            sendXmlErrorResponse(httpResponse, serviceResponse, ERROR_CANNOT_GET_SERVICE, sMsg);
            return ;
        }

        //------------------------------------------------------------------------
        //--- 2) Get the action
        //------------------------------------------------------------------------
        int iAction = ACTION_NONE;
        try
        {
            //--- Get the service action parameter
            String sAction = httpRequest.getParameter("action");
            if (sAction != null)
            {
                if (sAction.equalsIgnoreCase("get"))
                {
                    iAction = ACTION_GET;
                }
                else
                {
                    String sMsg = "Invalid action " + sAction  ;
                    error(sMsg);
                    ServiceResponse serviceResponse = new ServiceResponse(sServiceName, httpRequest, httpResponse, null );
                    sendXmlErrorResponse(httpResponse, serviceResponse, ERROR_INVALID_ACTION, sMsg);
                    return ;
                }
            }
        } catch (Throwable t)
        {
            String sMsg = "Cannot get service action (unexpected exception)"  ;
            error(sMsg, t);
            ServiceResponse serviceResponse = new ServiceResponse(sServiceName, httpRequest, httpResponse, null );
            serviceResponse.addException(t);                
            sendXmlErrorResponse(httpResponse, serviceResponse, ERROR_CANNOT_GET_ACTION, sMsg);
            return ;
        }

        //------------------------------------------------------------------------
        //--- 3) Build request environment 
        //------------------------------------------------------------------------
        ScreenSession   screenSession   = null;
        ServiceRequest  serviceRequest  = null;
        ServiceResponse serviceResponse = null;
        try
        {
            //--- Get the current "Screen Session" ( create a "dummy user" session if Authentication Not Requiered )
            // screenSession = ScreenRequestProcessor.getScreenSession(httpRequest); // v 1.1.0
            screenSession = ScreenSessionManager.getScreenSession(httpRequest); // v 1.1.0
            if ( null == screenSession )
            {
                String sMsg = "Cannot get screen session " ;
                error(sMsg);
                serviceResponse = new ServiceResponse(sServiceName, httpRequest, httpResponse, null );
                sendXmlErrorResponse(httpResponse, serviceResponse, ERROR_CANNOT_GET_SCREEN_SESSION, sMsg);
                return ;
            }

            //--- Build a Service Request instance
            serviceRequest = new ServiceRequest(sServiceName, httpRequest, screenSession );
 
            //--- Build a Service Response instance
            serviceResponse = new ServiceResponse(sServiceName, httpRequest, httpResponse, screenSession ); // v 0.9.9
            
        } catch (Throwable t)
        {
            String sMsg = "Cannot get service environment (unexpected exception)";
            error(sMsg, t);
            serviceResponse = new ServiceResponse(sServiceName, httpRequest, httpResponse, null );
            serviceResponse.addException(t);                
            sendXmlErrorResponse(httpResponse, serviceResponse, ERROR_CANNOT_GET_SERVICE_ENV, sMsg);
            return ;
        }

        //------------------------------------------------------------------------
        //--- 4) Execute the service
        //------------------------------------------------------------------------
        //--- The view where to forward (if any)
        View view = null;
        try
        {
            //--- Get the renderer
            sRenderer = oScreenService.getRenderer();
            
            //--- Get the session key (for session storage if any)
            sSessionKey = oScreenService.getSessionKey();
            trace("Service '" + sServiceName + "' found ( renderer = '" + sRenderer + "', session key = '" + sSessionKey + "' )");
            
            if ( iAction == ACTION_GET )
            {
                //------------------------------------------------------------------------
                //--- Execute the special "GET" action
                //------------------------------------------------------------------------
                doGetAction(screenSession, sSessionKey, serviceResponse);
            }
            else
            {
                //------------------------------------------------------------------------
                //--- Execute the service
                //------------------------------------------------------------------------
                try
                {
                    view = oScreenService.execute(screenSession, serviceRequest, serviceResponse);
                } catch (Throwable t)
                {
                    String sMsg = "Error in service " + sServiceName + " (exception)" ;
                    error(sMsg, t);
                    serviceResponse.addException(t);                
                    sendXmlErrorResponse(httpResponse, serviceResponse, ERROR_IN_SERVICE_EXECUTION, sMsg);
                    return ;
                }
            }
            
            //--- Store the result and the renderer in the Screen Session (only after "execute" not after "action" )
            if (iAction == ACTION_NONE && sSessionKey != null)
            {
                trace("Store service result in session (key=" + sSessionKey + ")");
                ServiceResultHolder srh = new ServiceResultHolder(oScreenService, serviceResponse);
                screenSession.setServiceResultHolder(sSessionKey, srh);
            }

            //--- Set the Service Response in the http request (to be used by renderer/view)
            httpRequest.setAttribute(SERVICE_RESPONSE, serviceResponse);

        } catch (Throwable t)
        {
            String sMsg = "Error in service " + sServiceName + " (exception)" ;
            error(sMsg, t);
            serviceResponse.addException(t);                
            sendXmlErrorResponse(httpResponse, serviceResponse, ERROR_IN_SERVICE_EXECUTION, sMsg);
        }

        //------------------------------------------------------------------------
        //--- 5) Generate the service response : view (NAV) or XML (RPC)
        //------------------------------------------------------------------------
        generateResponse(oScreenService, serviceResponse, view, httpRequest, httpResponse); // v 1.1.0
        
/*** 
        	//------------------------------------------------------------------------
            //--- Print the response ( View or Ajax response )
            //------------------------------------------------------------------------
            if (view != null)
            {
                //--- The service delegated the entire response management to an external VIEW
                trace("Service '" + sServiceName + "' : NAV : forward to view = '" + view + "' ...");
                view.generate(httpRequest, httpResponse);
                //--- Navigation notification
                navigation(httpRequest, oScreenService, view );
            }
            else
            {
                //--- The service generate the XML AJAX response by itself (using a partial view renderer, if any)
                trace("Service '" + sServiceName + "' : RPC : generate XML response ( renderer = '" + sRenderer + "' ) ...");
                
// ServiceRenderer removed in v 1.0.5
//                ServiceRenderer svcRenderer = null;
//                if (sRenderer != null)
//                {
//                    svcRenderer = new ServiceRenderer(sRenderer, httpRequest, httpResponse);
//                }
//                else
//                {
//                    svcRenderer = null;
//                }
                    
                //--- The response is in XML format + "charset UTF-8" +  BEFORE getWriter() 
                httpResponse.setContentType(TelosysConst.HTTP_CONTENT_TYPE_XML_UTF8); // v 1.0.5

                //--- Generate the response
                PrintWriter out = httpResponse.getWriter();

                //--- The response is in XML format 
                //httpResponse.setContentType(XML_CONTENT_TYPE); // v 1.0.5
                
//                XmlServiceOutput.generate(serviceResponse, out, TelosysConst.XML_PROLOG_ENCODING_UTF8, svcRenderer);
                ServiceXmlOutput.generate(serviceResponse, out, TelosysConst.XML_PROLOG_ENCODING_UTF8, sRenderer); // v 1.0.5
                
                out.close();
            }
***/
    }

    //---------------------------------------------------------------------------
    /**
     * Hook method designed to be overridden in a subclasse
     * @param request
     * @param service
     * @param view
     */
    protected void navigation(HttpServletRequest request, ScreenService service, View view )
    {
        // trace("navigation() : " + request.getRequestURI() +")");
    }
    
    //---------------------------------------------------------------------------
    /**
     * Generates an error response in XML format
     * 
     * @param response
     * @param pServiceResponse
     * @param iReturnCode
     * @param sMsg
     */
    private void sendXmlErrorResponse(HttpServletResponse response, ServiceResponse pServiceResponse, int iReturnCode, String sMsg) //throws ServletException, IOException
    {
        //--- Create a ScreenResponse object if it doesn't exist
        ServiceResponse serviceResponse = pServiceResponse;
        if (serviceResponse == null)
        {
            serviceResponse = new ServiceResponse("", null, null, null);
        }

        //--- Set the return code and message
        serviceResponse.setReturnCode(iReturnCode);
        serviceResponse.setReturnMessage(sMsg);

        //--- Send the HTTP Response
        try
        {
            //--- The response is in XML format + "charset UTF-8"  ( NB: must be done BEFORE getWriter() )
            response.setContentType(TelosysConst.HTTP_CONTENT_TYPE_XML_UTF8); // v 1.0.5

            PrintWriter out = response.getWriter();
            if (out == null)
            {
                error("Servlet output stream is null ");
                return;
            }

            //--- Generate the XML response
            ServiceXmlOutput.generate(serviceResponse, out, TelosysConst.XML_PROLOG_ENCODING_UTF8, null);

            out.close();

        } catch (Exception e)
        {
            //--- Log the error
            error("Error : " + e.getMessage());
        }
    }

    //---------------------------------------------------------------------------
    // Dtermination du nom de la classe  partir de l'URI de la requte HTTP
    // ( nom de classe uniquement sans le nom de package )
    //---------------------------------------------------------------------------
    private String getServiceNameFromURI(HttpServletRequest request)
    {
        String sURI = request.getRequestURI();
        int iFirst = sURI.lastIndexOf("/");
        if (iFirst < 0)
        {
            iFirst = 0;
        }
        else
        {
            iFirst++;
        }
        int iLast = sURI.lastIndexOf(".");
        if (iLast < 0)
        {
            return null;
        }
        return sURI.substring(iFirst, iLast);
    }

    //---------------------------------------------------------------------------
    private ScreenService getServiceFromProvider(String sServiceName) throws TelosysException
    {
        if (_serviceProvider == null)
        {
            //--- Get the Telosys factory
            TelosysFactory factory = Telosys.getFactory();
            if (factory == null)
            {
                throw new TelosysException("Cannot get the TelosysFactory !");
            }
            //--- Get the service provider
            _serviceProvider = factory.getServiceProvider();
            if (_serviceProvider == null)
            {
                throw new TelosysException("Cannot get the ServiceProvider from the TelosysFactory !");
            }
        }
        return _serviceProvider.getService(sServiceName);
    }

//    //---------------------------------------------------------------------------
//    private void executeService(ScreenSession screenSession, String sSessionKey, ServiceResponse serviceResponse)
//    {
//        view = oScreenService.execute(screenSession, serviceRequest, serviceResponse);
//
//    }
    
    //---------------------------------------------------------------------------
    private void generateResponse(ScreenService oScreenService, 
            ServiceResponse serviceResponse, View view, 
            HttpServletRequest httpRequest, HttpServletResponse httpResponse) // throws TelosysException
    {
        if (view != null)
        {
            //===== NAVIGATION 
            
            //--- The service delegated the entire response management to an external VIEW
            trace("Service '" + oScreenService.getName() + "' : NAV : forward to view = '" + view + "' ...");
            try
            {
                view.generate(httpRequest, httpResponse); // throws TelosysException

            } catch (Throwable e)
            {
                error("Exception in view generation (" + view +")", e);
            }

            try
            {
	            //--- Navigation notification
	            navigation(httpRequest, oScreenService, view ); // Overriden in application => catch all exceptions
            } catch (Throwable e)
            {
                error("Exception in navigation() hook method", e);
            }
            
        }
        else
        {
            //===== RPC 
            
            //--- Get the renderer
            String sRenderer = oScreenService.getRenderer();

            //--- The service generate the XML AJAX response by itself (using a partial view renderer, if any)
            trace("Service '" + oScreenService.getName() + "' : RPC : generate XML response ( renderer = '" + sRenderer + "' ) ...");
            
//ServiceRenderer removed in v 1.0.5
//            ServiceRenderer svcRenderer = null;
//            if (sRenderer != null)
//            {
//                svcRenderer = new ServiceRenderer(sRenderer, httpRequest, httpResponse);
//            }
//            else
//            {
//                svcRenderer = null;
//            }
                
            //--- The response is in XML format + "charset UTF-8"  ( NB: must be done BEFORE getWriter() )
            httpResponse.setContentType(TelosysConst.HTTP_CONTENT_TYPE_XML_UTF8); // v 1.0.5

            //--- Generate the response
            PrintWriter out = null ;
            try
            {
                out = httpResponse.getWriter();
            } catch (IOException e)
            {
                error("Servlet IOException on httpResponse.getWriter() : cannot generate the response", e);
                out = null ;
                return;
            }
            
            
            try
            {
                // XmlServiceOutput.generate(serviceResponse, out, TelosysConst.XML_PROLOG_ENCODING_UTF8, svcRenderer);
                ServiceXmlOutput.generate(serviceResponse, out, TelosysConst.XML_PROLOG_ENCODING_UTF8, sRenderer); // v 1.0.5
            } catch (TelosysException e1)
            {
                error("TelosysException in ServiceXmlOutput.generate", e1);
            } catch (Throwable e2)
            {
                error("Exception in ServiceXmlOutput.generate", e2);
            } finally 
            {
                out.close();
            }            
        }
        
    }
    
    //---------------------------------------------------------------------------
    private void doGetAction(ScreenSession screenSession, String sSessionKey, ServiceResponse serviceResponse)
    {
        trace("GET action : session key = '" + sSessionKey + "'");

        //--- Is there a result object stored with this KEY in the ScreenSession ?        
        ServiceResultHolder srh = screenSession.getServiceResultHolder(sSessionKey);
        if ( srh != null )
        {
            //--- Store the service result in the current service response (for the renderer)
            serviceResponse.setResult(srh.getResult());            
        }
        else
        {
            trace("Object NOT FOUND in session (key='" + sSessionKey + "').");            
        }
    }
}