package org.objectweb.telosys.rpl.xml.output;

/**
 * Error fired during the generation of the "data" section in the XML output.
 * 
 * 
 * @author Laurent Guerin
 * @since 1.0.5
 *
 */
public class DataOutputError extends OutputError
{
    private Object    _oDataInstance = null ;

    private String    _sElementName  = null ;
   
    /**
     * Constructor
     * @param oDataInstance
     * @param sElementName
     * @param exception
     */
    public DataOutputError(Object oDataInstance, String sElementName, Throwable exception )
    {
        super(exception);
        _oDataInstance = oDataInstance ;
        _sElementName = sElementName != null ? sElementName : "" ;
    }

    /**
     * Constructor
     * @param oDataInstance
     * @param sElementName
     * @param message
     */
    public DataOutputError(Object oDataInstance, String sElementName, String message)
    {
        super(message);
        _oDataInstance = oDataInstance ;
        _sElementName = sElementName != null ? sElementName : "" ;
    }
    
    /**
     * Returns the element name (never null)
     * @return
     */
    public String getElementName()
    {
        return _sElementName;
    }

    /**
     * Returns the data element instance 
     * @return
     */
    public Object getDataInstance()
    {
        return _oDataInstance;
    }

    /**
     * Returns the class of the data element instance 
     * @return
     */
    public Class getDataClass()
    {
        return _oDataInstance != null ? _oDataInstance.getClass() : null ;
    }

    /**
     * Returns the class name of the data element instance 
     * @return
     */
    public String getDataClassName()
    {
        return _oDataInstance != null ? _oDataInstance.getClass().getName() : null ;
    }
}
