package org.objectweb.telosys.rpl.xml.output;

/**
 * Holds an error that occurs durring the XML response generation 
 * 
 * @author Laurent Guerin
 * @since 1.0.5
 *
 */
public abstract class OutputError
{
    private Throwable _exception = null ;
    
    private String    _sMessage  = null ;
    
    /**
     * Constructor 
     * @param exception
     */
    public OutputError( Throwable exception )
    {
        super();
        _exception = exception;
        _sMessage  = null ;
    }

    /**
     * Constructor 
     * @param message
     */
    public OutputError( String message )
    {
        super();
        _exception = null;
        _sMessage  = message;
    }
    
    /**
     * Returns the exception that causes the error (or null if no exception)
     * @return
     */
    public Throwable getException()
    {
        return _exception;
    }
    
    /**
     * Returns the specific message if set, else the exception message if any. <br>
     * 
     * @return the message or "" (never null)
     */
    public String getMessage()
    {
        if ( _sMessage != null )  // 1) Specific message ( has priority )
        {
            return _sMessage ;
        }
        if ( _exception != null )  // 2) Exception message ( if any )
        {
            String s = _exception.getMessage() ;
            if ( s != null ) return s ;
        }
        return "";
    }
    
}
