package org.objectweb.telosys.rpl.xml.output;

import java.io.PrintWriter;
import java.util.Iterator;
import java.util.LinkedList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenRequestView;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.screen.core.ScreenDataElement;
import org.objectweb.telosys.screen.core.ScreenDataElements;

/**
 * Output generator for a Screen Action response
 *  
 * @author Laurent GUERIN
 *
 */
public class ScreenActionXmlOutput 
{
    private static final TelosysClassLogger log = new TelosysClassLogger(ScreenActionXmlOutput.class);
    
//	//--------------------------------------------------------------------------------------------------
//	// Mthodes de vrification des paramtres
//	//--------------------------------------------------------------------------------------------------
//	private static boolean checkPrintWriter(PrintWriter out)
//	{
//		if ( out == null )
//		{
//			log.error( "out parameter is null !" );
//			return false;
//		}
//		else
//		{
//			return true ;
//		}
//	}
//	//--------------------------------------------------------------------------------------------------
//    

	//--------------------------------------------------------------------------------------------------
	private static void beginningOfXml ( ScreenResponse resp, PrintWriter out)
	{
	    log.trace ( "beginningOfXml()" );
//		if ( checkPrintWriter(out) == false ) return ;
		
//		out.println("<?xml  version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
//		out.println("<?xml  version=\"1.0\" encoding=\"utf-8\" ?>");
		
		out.print("<response");
		out.print(" screen-name=\"" + resp.getScreenName() + "\"");
		out.print(" screen-id=\"" + resp.getScreenId() + "\"");
		out.print(" action=\"" + resp.getAction() + "\"" );
		String sProcName = resp.getProcName() ;
		if ( sProcName != null )
		{
			out.print(" proc-name=\"" + sProcName + "\"" );
		}
		out.println(" >");

	}

	//--------------------------------------------------------------------------------------------------
	// <return> ... </return>
	//--------------------------------------------------------------------------------------------------
	private static void genReturn ( ScreenResponse resp, PrintWriter out)
	{
	    log.trace ( "genReturn()" );
//		int iCount = 0 ;
//		if ( checkPrintWriter(out) == false ) return ;

		GenericXmlOutput.generateReturnOpeningTag(resp, out); // v 1.0.5
//		out.println("<return code=\"" + resp.getReturnCode() + "\" message=\"" + XmlUtil.xmlString( resp.getReturnMessage() ) + "\" >" );		            

		//--- Flag "FOUND" or "NOT-FOUND" (or nothing)
		if ( resp.isFlagNotFound() == true )
		{
			out.println(" <not-found/>" );
		}
		if ( resp.isFlagFound() == true )
		{
			out.println(" <found/>" );
		}
		
		GenericXmlOutput.generateExceptions( resp, out); // v 1.0.5
//		//--- List of exceptions 
//		iCount = resp.getNumberOfExceptions() ;
//		if ( iCount > 0 )
//		{
//		    //Exception ex = null ;
//		    Throwable ex = null ;		     
//			out.println("<exceptions>" );
//		    for ( int i = 0 ; i < iCount ; i++ )
//		    {
//		        ex = resp.getException(i);
//		        if ( ex != null )
//		        {
//					out.println(" <exception num=\"" + (i+1) +"\" class=\"" + ex.getClass().getName() + "\" message=\"" + XmlUtil.xmlString( ex.getMessage() ) + "\" />" );		            
//		        }
//		    }
//			out.println("</exceptions>" );
//		}

		GenericXmlOutput.generateErrors( resp, out); // v 1.0.5
//		//--- List of errors 
//		iCount = resp.getNumberOfErrors() ;
//		if ( iCount > 0 )
//		{
//		    ScreenError error = null ;
//			out.println("<errors>" );
//		    for ( int i = 0 ; i < iCount ; i++ )
//		    {
//		        error = resp.getError(i);
//		        if ( error != null )
//		        {
//					out.println(" <error num=\"" + (i+1) +"\" code=\"" + error.getCode() + "\" text=\"" + XmlUtil.xmlString( error.getText() ) + "\" />" );		            
//		        }
//		    }
//			out.println("</errors>" );
//		}

		GenericXmlOutput.generateMessages( resp, out); // v 1.0.5
//		//--- List of messages 
//		iCount = resp.getNumberOfMessages() ;
//		if ( iCount > 0 )
//		{
//		    ScreenMessage message = null ;
//			out.println("<messages>" );
//		    for ( int i = 0 ; i < iCount ; i++ )
//		    {
//		        message = resp.getMessage(i);
//		        if ( message != null )
//		        {
//					out.println(" <message num=\"" + (i+1) 
//					        + "\" type=\"" + message.getType()
//					        + "\" code=\"" + message.getCode()
//					        + "\" text=\"" + XmlUtil.xmlString( message.getText() ) + "\" />" );		            
//		        }
//		    }
//			out.println("</messages>" );
//		}

		GenericXmlOutput.generateReturnClosingTag(out); // v 1.0.5
//		out.println("</return>" );
	}
	
//	//--------------------------------------------------------------------------------------------------
//	// <values> ... </values>
//	//--------------------------------------------------------------------------------------------------
//	private static void genValues ( ScreenResponse resp, PrintWriter out)
//	{
//	    log.trace ( "genValues()" );
//		if ( checkPrintWriter(out) == false ) return ;
//		if ( resp.getNumberOfValues() > 0 )
//		{
//		    String sValueName = null ;
//			Enumeration eNames = resp.getValueNames();
//			if ( eNames != null )
//			{
//				out.println("<values>" );
//			    while ( eNames.hasMoreElements() )
//			    {
//			        sValueName = (String) eNames.nextElement() ;
//					out.println("<value " + sValueName + "=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
//			    }
//				out.println("</values>" );			    
//			}
//		}
//	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates a tag in the "data" section for each element of the given elements list
	 * @param elements
	 * @param out
	 * @param wrapperProvider
	 * @throws TelosysException
	 */
	private static LinkedList genDataElements ( ScreenDataElements elements, PrintWriter out) //throws TelosysException
	{
	    log.trace ( "genDataElements()" );

// Removed in v 1.0.5
//	    IXmlWrapperProvider wrapperProvider = Telosys.getXmlWrapperProvider() ;
//        if (wrapperProvider == null)
//		{
//            throw new TelosysRuntimeException("toXml : cannot get XmlWrapperProvider ");
//		}
	    
	    LinkedList dataErrors = null;

		//--- Get the elements list 
        if ( elements != null ) 
        {
            //--- Generate a tag in the "<data> section" for each element 
            if ( elements.size() > 0 )
            {
                ScreenDataElement elem = null ;
        		out.println("<data>" );
        		Iterator iter = elements.iterator() ;
        		while ( iter.hasNext() )
        		{
        		    elem = (ScreenDataElement) iter.next();
        		    if ( elem != null )
        		    {
        		        // elem.toXmlData(out, wrapperProvider);   // removed in v 1.0.5    		            
        		        Object obj  = elem.getInstance();  //  v 1.0.5
        		        String name = elem.getName(); //  v 1.0.5
        		        DataOutputError err = GenericXmlOutput.generateDataElement(obj, name, out); //  v 1.0.5
        		        //--- Keep the error in the errors list
			            if ( err != null )
			            {
			                if ( null == dataErrors )
			                {
			                    dataErrors = new LinkedList();
			                }
			                dataErrors.add(err);
			            }

        		    }
        		}
        		out.println("</data>" );         
            }
        }
        return dataErrors ;
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void genDataVoid ( PrintWriter out ) //throws TelosysException
	{
	    log.trace ( "genDataVoid()" );
//		if ( checkPrintWriter(out) == false ) return ;
		out.println("<data void=\"true\" />" );
	}
	
	//--------------------------------------------------------------------------------------------------
	private static LinkedList genViews ( ScreenRequestView[] views, PrintWriter out, HttpServletRequest httpRequest, HttpServletResponse httpResponse) //throws TelosysException
	{
	    log.trace ( "genViews()" );
//		if ( checkPrintWriter(out) == false ) return ;

	    LinkedList viewErrors = null;
		if ( views != null )
		{
		    if ( views.length > 0 )
		    {		        
	    		out.println("<view>" );
	    		//--- For each VIEW : generate the view using the associated renderer
				for ( int i=0 ; i < views.length ; i++ )
				{
				    ScreenRequestView view = views[i];
			        if ( view != null )
			        {
			            String sName = view.getName() ;
			            String sRenderer = view.getRenderer() ;
			            
			            ViewOutputError err = GenericXmlOutput.generateViewContent( out, sName, sRenderer, httpRequest, httpResponse ); // v 1.0.5
        		        //--- Keep the error in the errors list
			            if ( err != null )
			            {
			                if ( null == viewErrors )
			                {
			                    viewErrors = new LinkedList();
			                }
			                viewErrors.add(err);
			            }
			            
//			    		out.println("<"+ sName + " renderer=\"" + sRenderer + "\" >");    		
//			    		out.println("<![CDATA[");    	
//			    		//--- Render the view content 
//			    		view.generate(httpRequest, httpResponse); 
//	    	    		out.println("]]>");    		
//			    		out.println("</"+ sName +">");    		
			        }			    
				}		    
	    		out.println("</view>" );
	    		
		    }
		}
		return viewErrors ;
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void endOfXml ( PrintWriter out ) 
	{
	    log.trace ( "endOfXml()" );
//		if ( checkPrintWriter(out) == false ) return ;
		out.println("</response>");
	}
	//--------------------------------------------------------------------------------------------------
    
    /**
     * Generates the XML response flow for the current ACTION REQUEST
     * @param screenRequest
     * @param screenResponse
     * @param out
     * @param httpRequest
     * @param httpResponse
     * @throws TelosysException
     */
    public final static void generate ( ScreenRequest screenRequest, ScreenResponse screenResponse, 
            PrintWriter out, HttpServletRequest httpRequest, HttpServletResponse httpResponse ) //throws TelosysException
    {
        log.trace ( "generate()" );

		if ( screenRequest == null )
		{
			log.error( "request parameter is null !" );
			return ;
		}
		if ( screenResponse == null )
		{
			log.error( "response parameter is null !" );
			return ;
		}
		if ( out == null )
		{
			log.error( "out parameter is null !" );
			return ;
		}
        
        beginningOfXml ( screenResponse, out);
        genReturn ( screenResponse, out);
        
        GenericXmlOutput.generateValues(screenResponse, out); // v 1.0.5
//        genValues ( screenResponse, out);
        
        //--- If the response status is OK : generate ELEMENTS and VIEWS (if any) 
        if ( screenResponse.getReturnCode() == ScreenResponse.ACTION_OK )
        {            
            if ( screenRequest.getFlagGetData() == true )
            {
                LinkedList dataErrors = null ;
                LinkedList viewErrors = null ;
                
		        //--- Generate data elements 
	            ScreenDataElements elements = screenResponse.getDataElements();             
		        if ( elements != null )
		        {
		            genDataElements ( elements, out);
		            // TODO
		            //dataErrors = genDataElements ( elements, out);
		            dataErrors = null ; 
		        }
		        else
		        {
		            // No element 
		            genDataVoid (out);
		            dataErrors = null ; 
		        }
		        
		        //--- Generate views
		        ScreenRequestView[] views = screenRequest.getViews() ; 
		        if ( views != null )
		        {
		            viewErrors = genViews(views, out, httpRequest, httpResponse);
		        }
		        
	    		GenericXmlOutput.generateOutputErrors(dataErrors, viewErrors, out );

            }
            else
            {
	            genDataVoid (out); // get="false" => Do not return elements               
            }
        }
        
        endOfXml (out);
        out.flush();
    }
    
	//--------------------------------------------------------------------------------------------------
    /**
     * Generates the XML response flow for an ACTION ERROR
     * @param response
     * @param out
     */
    public final static void generateError ( ScreenResponse response, PrintWriter out ) 
    {
        beginningOfXml ( response, out);
        genReturn ( response, out);        
        endOfXml (out);
        out.flush();        
    }
}
