package org.objectweb.telosys.rpl.xml.output;

import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.service.ServiceResponse;

/**
 * Output generator for a SERVICE response
 * 
 * @author Laurent GUERIN
 *
 */
public class ServiceXmlOutput 
{
    private static final TelosysClassLogger log = new TelosysClassLogger(ServiceXmlOutput.class);
    
    private static final String RESULT_TAG_NAME = "service-result" ;
    
    private static final String VIEW_TAG_NAME   = "service-view" ;
    
//	//--------------------------------------------------------------------------------------------------
//	// Mthodes de vrification des paramtres
//	//--------------------------------------------------------------------------------------------------
//	private static boolean checkPrintWriter(PrintWriter out)
//	{
//		if ( out == null )
//		{
//			log.error( "out parameter is null !" );
//			return false;
//		}
//		else
//		{
//			return true ;
//		}
//	}
//	//--------------------------------------------------------------------------------------------------
    

	//--------------------------------------------------------------------------------------------------
	private static void beginningOfXml ( ServiceResponse resp, PrintWriter out, String encoding )
	{
	    log.trace ( "beginningOfXml()" );
//		if ( checkPrintWriter(out) == false ) return ;
		
		//out.println("<?xml  version=\"1.0\" encoding=\"ISO-8859-1\" ?>");  // v 1.0.5
		if ( encoding != null )
		{
			out.println("<?xml version=\"1.0\" encoding=\"" + encoding + "\" ?>");
		}
		
		out.print("<response");
		out.print(" service-name=\"" + resp.getServiceName() + "\"");
		out.println(" >");

	}

	//--------------------------------------------------------------------------------------------------
	// <return> ... </return>
	//--------------------------------------------------------------------------------------------------
	private static void genReturn ( ServiceResponse resp, PrintWriter out)
	{
	    log.trace ( "genReturn()" );
//		int iCount = 0 ;
//		if ( checkPrintWriter(out) == false ) return ;

		GenericXmlOutput.generateReturnOpeningTag(resp, out); // v 1.0.5
//		out.println("<return code=\"" + resp.getReturnCode() + "\" message=\"" + XmlUtil.xmlString( resp.getReturnMessage() ) + "\" >" );		            
		
		GenericXmlOutput.generateExceptions( resp, out); // v 1.0.5
//		//--- List of exceptions 
//		iCount = resp.getNumberOfExceptions() ;
//		if ( iCount > 0 )
//		{
//		    Exception ex = null ;
//			out.println("<exceptions>" );
//		    for ( int i = 0 ; i < iCount ; i++ )
//		    {
//		        ex = resp.getException(i);
//		        if ( ex != null )
//		        {
//					out.println(" <exception num=\"" + (i+1) +"\" class=\"" + ex.getClass().getName() + "\" message=\"" + XmlUtil.xmlString( ex.getMessage() ) + "\" />" );		            
//		        }
//		    }
//			out.println("</exceptions>" );
//		}

		GenericXmlOutput.generateErrors( resp, out); // v 1.0.5
//		//--- List of errors 
//		iCount = resp.getNumberOfErrors() ;
//		if ( iCount > 0 )
//		{
//		    ScreenError error = null ;
//			out.println("<errors>" );
//		    for ( int i = 0 ; i < iCount ; i++ )
//		    {
//		        error = resp.getError(i);
//		        if ( error != null )
//		        {
//					out.println(" <error num=\"" + (i+1) +"\" code=\"" + error.getCode() + "\" text=\"" + XmlUtil.xmlString( error.getText() ) + "\" />" );		            
//		        }
//		    }
//			out.println("</errors>" );
//		}

		GenericXmlOutput.generateMessages( resp, out); // v 1.0.5
//		//--- List of messages 
//		iCount = resp.getNumberOfMessages() ;
//		if ( iCount > 0 )
//		{
//		    ScreenMessage message = null ;
//			out.println("<messages>" );
//		    for ( int i = 0 ; i < iCount ; i++ )
//		    {
//		        message = resp.getMessage(i);
//		        if ( message != null )
//		        {
//					out.println(" <message num=\"" + (i+1) 
//					        + "\" type=\"" + message.getType()
//					        + "\" code=\"" + message.getCode()
//					        + "\" text=\"" + XmlUtil.xmlString( message.getText() ) + "\" />" );		            
//		        }
//		    }
//			out.println("</messages>" );
//		}

		GenericXmlOutput.generateReturnClosingTag(out); // v 1.0.5
//		out.println("</return>" );
	}
	
//	//--------------------------------------------------------------------------------------------------
//	// <values> ... </values>
//	//--------------------------------------------------------------------------------------------------
//	private static void genValues ( ServiceResponse resp, PrintWriter out)
//	{
//	    log.trace ( "genValues()" );
//		if ( checkPrintWriter(out) == false ) return ;
//		if ( resp.getNumberOfValues() > 0 )
//		{
//		    String sValueName = null ;
//			Enumeration eNames = resp.getValueNames();
//			if ( eNames != null )
//			{
//				out.println("<values>" );
//			    while ( eNames.hasMoreElements() )
//			    {
//			        sValueName = (String) eNames.nextElement() ;
//					//OLD: out.println(" <" + sValueName + " value=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
//					out.println("<value " + sValueName + "=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
//			    }
//				out.println("</values>" );			    
//			}
//		}
//	}

	private static void printError ( PrintWriter out, Throwable ex)
	{
	    if ( ex != null ) // v 1.0.2
	    {	        
	        out.println("<b>Exception</b> : " + ex.getClass().getName() + " ( " + ex.getMessage() + " ) <br>");
            if ( ex instanceof ServletException )
            {
                //--- ServletException : get "Root Cause"
                ServletException ex2 = (ServletException) ex ;
                Throwable cause = ex2.getRootCause();
                if ( cause != null )
                {
	                out.println("<b>Root Cause</b> : <br>");
	                printError ( out, cause );                
                }
                else
                {
	                out.println("<b>No root cause</b><br>");
                }
            }
            else
            {
                //--- Other Exception : get "Standard Cause"
                Throwable cause = ex.getCause();
                if ( cause != null )
                {
	                out.println("<b>Cause</b> : <br>");
	                printError ( out, cause );                
                }
                else
                {
	                out.println("<b>No cause</b><br>");
                }
            }
	    }
	}
	
//	//--------------------------------------------------------------------------------------------------
//	private static void genView ( ServiceResponse response, PrintWriter out, ServiceRenderer renderer)// throws TelosysException
//	{
//		out.println("<view>");
//		out.println("<service-view>");
//		out.println("<![CDATA[");    		
//        try
//        {
//            renderer.render(response);
//        } catch (Throwable e)
//        {
//            out.println("<b>ERROR</b> in renderer <b>'" + renderer.getPath() + "'</b><br>");
//            printError(out, e);
////            out.println("<b>Exception</b> : " + e.getClass().getName() + "<br>");
////            out.println(" . Message : " + e.getMessage() + "<br>" );
////            out.println(" . Cause : " + e.getCause() + "<br>" );
//        }
//        finally {
//    		out.println("]]>");    		
//    		out.println("</service-view>");
//    		out.println("</view>");                            
//        }
//	}
	
	//--------------------------------------------------------------------------------------------------
	private static ViewOutputError genView ( ServiceResponse response, PrintWriter out, String sRenderer)// throws TelosysException
	{
	    HttpServletRequest httpRequest = response.getHttpRequest();
	    HttpServletResponse httpResponse = response.getHttpResponse() ;
	    
	    //--- 1) Set the "RENDERER BEAN" in the Request  // v 1.1.0.b
	    if ( response.getResult() != null ) // v 1.1.0.b
	    {
		    httpRequest.setAttribute( TelosysConst.RENDERER_BEAN, response.getResult() ); // v 1.1.0.b
	    }
	    
	    //--- 2) Generate the VIEW using the RENDERER
		out.println("<view>");
	    ViewOutputError viewError = GenericXmlOutput.generateViewContent( out, VIEW_TAG_NAME, sRenderer, httpRequest, httpResponse ); // v 1.0.5
		out.println("</view>");
		
		return viewError ;
//		GenericXmlOutput.generateViewErrors(viewError, out ); // v 1.0.5
	}

	//--------------------------------------------------------------------------------------------------
	private static DataOutputError genData ( ServiceResponse serviceResponse, PrintWriter out) //throws TelosysException
	{
	    log.trace ( "genData()" );
//		if ( checkPrintWriter(out) == false ) return ;
	    
	    DataOutputError dataError = null ;
		out.println("<data>" );
	    Object oResult = serviceResponse.getResult() ;
	    if ( oResult != null )
	    {

// removed in v 1.0.5	        
//	        // Try to generate a generic object ( VOList, DataSet )
//	        boolean bGenerated = genGenericObject ( oResult, out) ;
//	        if ( ! bGenerated )
//	        {
//		        // Not yet generated => Try to generate as a standard Java object 
//	            bGenerated = genJavaObject ( oResult, out) ;
//	        }
//	        if ( ! bGenerated )
//	        {
//		        // Not yet generated => Try to generate as a specific object ( with specific mapper )
//	            genSpecificObject ( oResult, out) ;
//	        }
	        
	        dataError = GenericXmlOutput.generateDataElement(oResult, RESULT_TAG_NAME, out); //  v 1.0.5
	        
	    }
	    else
	    {
    		out.println("<!-- No result object -->" );
	    }
		out.println("</data>" );     
		return dataError ;
	}

//	//--------------------------------------------------------------------------------------------------
//	private static boolean genJavaObject ( Object oResult, PrintWriter out) 
//	{
//	    String sClassName = oResult.getClass().getName() ;
//	    //if ( sClassName.startsWith("java.lang") || sClassName.startsWith("java.math")  )
//		if ( sClassName.startsWith("java.") ) // java.lang, java.math, java.util 
//	    {
//	        JavaObjectWriter.toXml(out, oResult, RESULT_TAG_NAME);
//	        return true ;
//	    }
//	    return false ;
//	}
	
//	//--------------------------------------------------------------------------------------------------
//	private static boolean genGenericObject ( Object oResult, PrintWriter out) 
//	{
//	    if ( oResult instanceof GenericVOList )
//	    {
//	        GenericVOListWriter.toXml(out, (GenericVOList) oResult, RESULT_TAG_NAME, "item" );
//	        return true ;
//	    }
//	    else if ( oResult instanceof DataSet )
//	    {
//	        DataSetWriter.toXml(out, (DataSet) oResult, RESULT_TAG_NAME );
//	        return true ;
//	    }
//	    return false ;
//	}
	
//	//--------------------------------------------------------------------------------------------------
//	private static boolean genSpecificObject ( Object oResult, PrintWriter out) //throws TelosysException
//	{
//	    boolean bRet = false ;
//	    IXmlWrapperProvider wrapperProvider = Telosys.getXmlWrapperProvider() ;
//        if (wrapperProvider == null)
//		{
//    		out.println("<!-- ERROR : cannot get XmlWrapperProvider -->" );
//            //throw new TelosysRuntimeException("toXml : cannot get XmlWrapperProvider ");
//		}
//        else
//        {
//	        //--- Get the XML Wrapper for the given object
//            IXmlWrapper wrapper = null;
//            try
//            {
//                wrapper = wrapperProvider.getWrapper(oResult);
//            } catch (TelosysRuntimeException e)
//            {
//                wrapper = null;
//            }
//	        if (wrapper != null)
//	        {
//	            //--- Print the XML tag
//	            wrapper.toXml(out, oResult, RESULT_TAG_NAME);
//	            bRet = true ;
//	        }
//	        else
//	        {
//	    		out.println("<!-- Cannot get mapper for class " + oResult.getClass().getName() + " -->" );
//	        }
//        }
//        return bRet ;
//	}
	
	//--------------------------------------------------------------------------------------------------
	private static void endOfXml ( PrintWriter out ) 
	{
	    log.trace ( "endOfXml()" );
//		if ( checkPrintWriter(out) == false ) return ;
		out.println("</response>");
	}
	//--------------------------------------------------------------------------------------------------
    
    /**
     * Generates the XML response flow for the given service response
     * @param response the service response object
     * @param out
     * @param encoding the encoding to set in the XML prolog ( e.g. "UTF-8" ), if null the XML prolog is not generated
     * @param renderer the renderer to use, or null if none
     * @throws TelosysException
     */
    public final static void generate ( ServiceResponse response, PrintWriter out, String encoding, String renderer ) throws TelosysException
    {
        log.trace ( "generate()" );
        
		if ( out == null )
		{
			log.error( "out parameter is null !" );
			return ;
		}
		if ( response == null )
		{
			log.error( "response parameter is null !" );
			return ;
		}

        beginningOfXml ( response, out, encoding );
        genReturn ( response, out);
        
        GenericXmlOutput.generateValues(response, out); // v 1.0.5
//        genValues ( response, out);
        
        if ( renderer != null )
        {
            // There's a renderer for this service => use it to generate a View
            ViewOutputError viewError = genView(response, out, renderer );
            GenericXmlOutput.generateOutputErrors(null, viewError, out); // v 1.0.5
        }
        else
        {
            // If there's no renderer, generate the data in raw format (XML)
            DataOutputError dataError = genData(response, out );
            GenericXmlOutput.generateOutputErrors(dataError, null, out); // v 1.0.5
        }
        
        endOfXml (out);
        out.flush();
    }
}
