package org.objectweb.telosys.screen.core;

import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenResponse;

/**
 * The ScreenTriggers interface <br>
 * Defines all the methods that must be implemented by a SCREEN TRIGGERS MANAGER <br>
 * <br>
 * Implemented in the framework by "StandardScreenTriggers" <br>
 * 
 * @author Laurent GUERIN
 *
 */
public interface ScreenTriggers
{
    //--- SET 
    /**
     * Method triggered before a "set" action
     * 
     * @param dataToSet
     * @param screen
     * @param request
     * @param response
     * @return
     */
    public boolean beforeSet(ScreenData dataToSet, ScreenContext screen, ScreenRequest request, ScreenResponse response );

    /**
     * Method triggered after a "set" action
     * 
     * @param dataSet
     * @param screen
     * @param request
     * @param response
     * @return
     */
    public boolean afterSet(ScreenData dataSet, ScreenContext screen, ScreenRequest request, ScreenResponse response );

    //--- GET 
    /**
     * Method triggered before a "get" action
     * 
     * @param screen
     * @param request
     * @param response
     * @return
     */
    public boolean beforeGet( ScreenContext screen, ScreenRequest request, ScreenResponse response );

    /**
     * Method triggered after a "get" action
     * 
     * @param screen
     * @param request
     * @param response
     * @return
     */
    public boolean afterGet( ScreenContext screen, ScreenRequest request, ScreenResponse response );
    
    //--- CLEAR 
    /**
     * Method triggered before a "clear" action
     * 
     * @param screen
     * @param request
     * @param response
     * @return
     */
    public boolean beforeClear( ScreenContext screen, ScreenRequest request, ScreenResponse response );

    /**
     * Method triggered after a "clear" action
     * 
     * @param screen
     * @param request
     * @param response
     * @return
     */
    public boolean afterClear( ScreenContext screen, ScreenRequest request, ScreenResponse response );
    
    
    //--- LOAD 
    /**
     * Method triggered before a "load" action
     * 
     * @param dataForLoad
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean beforeLoad(ScreenData dataForLoad, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

    /**
     * Method triggered after a "load" action
     * 
     * @param dataLoaded
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean afterLoad(ScreenData dataLoaded, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

    //--- SAVE 
    /**
     * Method triggered before a "save" action
     * 
     * @param dataToSave
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean beforeSave(ScreenData dataToSave, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

    /**
     * Method triggered after a "save" action
     * 
     * @param dataSaved
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean afterSave(ScreenData dataSaved, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

    //--- INSERT 
    /**
     * Method triggered before an "insert" action
     * 
     * @param dataToInsert
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean beforeInsert(ScreenData dataToInsert, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

    /**
     * Method triggered after an "insert" action
     * 
     * @param dataInserted
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean afterInsert(ScreenData dataInserted, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

    //--- UPDATE 
    /**
     * Method triggered before an "update" action
     * 
     * @param dataToUpdate
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean beforeUpdate(ScreenData dataToUpdate, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

    /**
     * Method triggered after an "update" action
     * 
     * @param dataUpdated
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean afterUpdate(ScreenData dataUpdated, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

    //--- DELETE 
    /**
     * Method triggered before a "delete" action
     * 
     * @param dataToDelete
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean beforeDelete(ScreenData dataToDelete, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

    /**
     * Method triggered after a "delete" action
     * 
     * @param dataDeleted
     * @param screen
     * @param request
     * @param response
     * @param session
     * @return
     */
    public boolean afterDelete(ScreenData dataDeleted, ScreenContext screen, ScreenRequest request, ScreenResponse response, DatabaseSession session);

}
