package org.objectweb.telosys.screen.core;

import javax.servlet.ServletContext;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.dal.TelosysDAL;
import org.objectweb.telosys.dal.dao.DAOProvider;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.dal.dao.StandardDAO;
import org.objectweb.telosys.dal.sql.ConnectionManager;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenApplicationManager;

/**
 * Common part for the following classes :  <br>
 * . StandardScreenManager <br>
 * . StandardScreenProcedures <br>
 * . StandardscreenTriggers <br>
 * 
 * @author Laurent GUERIN
 * @since 1.1.0
 *  
 */
/* package */ abstract class StandardScreenCommon extends TelosysObject
{

    //----------------------------------------------------------------------------------------------
    /**
     * Returns the ScreenApplication object
     * @return
     */
    protected final ScreenApplication getScreenApplication() 
    {
        return ScreenApplicationManager.getScreenApplication();
    }
    
    //--------------------------------------------------------------------------------------------------------------
    /**
     * Returns the ServletContext object
     * @return
     */
    protected final ServletContext getServletContext() 
    {
        return ScreenApplicationManager.getServletContext();
    }
    

    //--------------------------------------------------------------------------------------------------------------
	/**
	 * Returns the DAO for the given bean class and the default database
	 * @param beanClass
	 * @return
	 * @throws TelosysException
	 * @since 1.1.0
	 */
	protected final StandardDAO getDAO(Class beanClass)  throws TelosysException
	{
		return getDAO(beanClass, ConnectionManager.getDefaultDatabase() );
	}
    
    //--------------------------------------------------------------------------------------------------------------
	/**
	 * Returns the DAO for the given bean class and db session
	 * @param beanClass
	 * @param dbSession
	 * @return
	 * @throws TelosysException
	 * @since 1.0.2
	 */
	protected final StandardDAO getDAO(Class beanClass, DatabaseSession dbSession)  throws TelosysException
	{
		if ( dbSession == null )
		{
			throw new TelosysException("getDAO() : DatabaseSession parameter is null" );
		}
		return getDAO(beanClass, dbSession.getDatabaseId() );
	}
	
    //--------------------------------------------------------------------------------------------------------------
	/**
	 * Returns the DAO for the given bean class and db id
	 * @param beanClass
	 * @param iDatabaseId
	 * @return
	 * @throws TelosysException
	 * @since 1.0.2
	 */
	protected final StandardDAO getDAO(Class beanClass, int iDatabaseId)  throws TelosysException
	{
		if ( beanClass == null )
		{
			throw new TelosysException("getDAO() : bean class parameter is null" );
		}
		DAOProvider provider = TelosysDAL.getDAOProvider();
		if ( provider == null )
		{
			throw new TelosysException("getDAO() : cannot get DAO provider" );
		}
		StandardDAO dao = provider.getDAO(beanClass, iDatabaseId);
		if ( dao != null )
		{
			return dao ;
		}
		else
		{
			throw new TelosysException("No DAO for for class '" + beanClass.getName() + "' and database '" + iDatabaseId + "'" );
		}
	}
    
}