package org.objectweb.telosys.service;

import org.objectweb.telosys.common.TelosysException;


/**
 * The "service provider" interface 
 * 
 * @author Laurent GUERIN
 *
 */
public interface IServiceProvider  
{
    /**
     * Returns an instance of the service identified by its name
     * @param sServiceName the name of the service
     * @return the service instance
     * @throws TelosysException
     */
    public ScreenService getService ( String sServiceName ) throws TelosysException ;
    
    /**
     * Returns the key used to store the service result in the ScreenSession 
     * @param sServiceName the name of the service
     * @return the key (can be null if no result storage)
     * @throws TelosysException
     */
    public String getSessionKey(String sServiceName) throws TelosysException ;

    
    /**
     * Initializes the screen provider using the current Telosys properties
     * 
     * @since 1.0.5
     */
    public void init();
    
}
