package org.objectweb.telosys.service;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * Static methods for Service parameters management
 * 
 * @author Laurent Guerin
 * @since  1.0.5
 *
 */
public class ParametersManager
{
    //-----------------------------------------------------------------------------
    /**
     * Converts all the parameters of the given map from ISO-8859-1 to UTF-8
     * 
     * @param map
     * @return a new Map instance containing the converted parameters (or null if the given map is null)
     */
    protected static Map convertParametersToUTF8 ( Map map )
    {
    	if ( null == map )
    	{
    		return null ;
    	}
    	if ( map.isEmpty() )
    	{
    		return map ;
    	}

    	Map map2 = new HashMap();
    	
    	Set keys = map.keySet();
        if (keys != null)
        {
        	Iterator iter = keys.iterator();
        	while ( iter.hasNext() )
        	{
        		String name = (String) iter.next();
        		String[] values = (String[]) map.get(name);
        		if ( values != null )
                {
        			// Convert all the values to UTF-8
        			String[] values2 = new String [ values.length ];
                    for ( int i=0 ; i < values.length ; i++ )
                    {
                        values2[i] = convertToUTF8( values[i] );
                    }
            		map2.put(name, values2);
                }
        	}
        }
    	return map2;
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Converts the given ISO-8859-1 string to UTF-8 string
     * @param s
     * @return
     */
    protected static String convertToUTF8 ( String s )
    {
    	String s2 = s ;
    	try 
    	{
			return new String(s.getBytes("iso-8859-1"), "UTF-8");
		} 
    	catch (UnsupportedEncodingException e) // Never happends
    	{
			s2 = s ; 
		} 
		return s2 ;
    }
    //----------------------------------------------------------------------------- 
}
