package org.objectweb.telosys.service;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.View;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * The ScreenService interface
 * 
 * @author Laurent GUERIN
 *
 */
public interface ScreenService
{
    /**
     * Initializes the service instance (just after creation)
     * @param sServiceName the name of the service
     * @param sRenderer the renderer associated with the service (or null if none)
     * @param sSessionKey the session key to use for result storage (or null if no storage)
     */
    public void init ( String sServiceName, String sRenderer, String sSessionKey ) ;
    
    /**
     * Executes the service 
     * @param screenSession the user's screen session
     * @param serviceRequest the service request instance
     * @param serviceResponse the service response instance
     * @return
     * @throws TelosysException
     */
    public View execute ( ScreenSession screenSession, ServiceRequest serviceRequest, ServiceResponse serviceResponse ) throws TelosysException ;
    
    /**
     * Returns the renderer associated with the service (or null if none) 
     * @return
     */
    public String getRenderer() ;

    /**
     * Returns the name of the service
     * @return
     */
    public String getName() ;
    
    /**
     * Returns the session key (or null if none) 
     * @return
     */
    public String getSessionKey() ;
    
}
