package org.objectweb.telosys.uil;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.TelosysObject;

/**
 * This object holds the fields values of a screen, in order to reuse them for an automatic postback  
 * 
 * @author Laurent GUERIN
 *  
 */
public class ScreenPostback extends TelosysObject implements Serializable
{
    /**
     * <code>_htData</code> contains the field values <br>
     * The key is FORM NAME + "." + FIELD NAME
     */
    private Hashtable _htData = new Hashtable();

    
    /**
     * Constructor
     */
    public ScreenPostback()
    {
        super();
    }
    
    /**
     * If the string is null or contains only spaces : returns null, else returns the string after "trim"
     * 
     * @param s
     * @return the significant string
     */
    private String getSignificant(String s)
    {
        if (s == null)
        {
            return null;
        }
        else
        {
            String s2 = s.trim();
            if (s2.length() > 0)
            {
                return s2;
            }
            else
            {
                return null;
            }
        }
    }

    /**
     * Composes the HashTable key : FORM NAME + "." + FIELD NAME
     * @param sArgFormName
     * @param sArgFieldName
     * @return
     */
    private String composeFieldKey(String sArgFormName, String sArgFieldName)
    {
        String sFormName = getSignificant(sArgFormName);
        String sFieldName = getSignificant(sArgFieldName);
        if (sFormName != null && sFieldName != null)
        {
            return sFormName + "." + sFieldName;
        }
        else if (sFormName == null && sFieldName != null)
        {
            return sFieldName;
        }
        else
        {
            return "(no_field_key)";
        }
    }

    /**
     * Removes all the field values for the given FORM NAME
     * @param sFormId
     */
    private void clearFormValues(String sFormId)
    {
        trace("clearFormValues(" + sFormId + ")");
        if (sFormId != null)
        {
            String sPrefix = sFormId.trim() + ".";
            String sKey = null;
            Enumeration enKeys = _htData.keys();
            if (enKeys != null)
            {
                while (enKeys.hasMoreElements())
                {
                    sKey = (String) enKeys.nextElement();
                    if (sKey.startsWith(sPrefix))
                    {
                        _htData.remove(sKey);
                    }
                }
            }
        }
    }

    /**
     * Store each request parameter in the HashTable 
     * @param sFormName
     * @param httpRequest
     */
    private void storeFormValues(String sFormName, HttpServletRequest httpRequest)
    {
        trace("storeFormValues(" + sFormName + ", httpRequest)");
        //String sPrefix = sFormId + ".";
        Map parameters = httpRequest.getParameterMap();
        if (parameters != null)
        {
            if (!parameters.isEmpty())
            {
                Set keys = parameters.keySet();
                Iterator it = keys.iterator();
                if (it != null)
                {
                    while (it.hasNext())
                    {
                        String sFieldName = (String) it.next();
                        String sFieldKey = composeFieldKey(sFormName, sFieldName);
                        trace("store field values : key = '" + sFieldKey + "' ");
                        //--- Store the String array of values for this field
                        _htData.put(sFieldKey, parameters.get(sFieldName));
                    }
                }
            }
        }
    }

    /**
     * Initializes the field values with the parameters received in the HTTP request 
     * @param sFormName
     * @param httpRequest
     */
    public void initFieldValues(String sFormName, HttpServletRequest httpRequest)
    {
        trace("initFieldValues(" + sFormName + ", httpRequest)");
        if (sFormName == null)
        {
            sFormName = "";
        }
        sFormName = sFormName.trim();
        clearFormValues(sFormName);
        storeFormValues(sFormName, httpRequest);

        //        for ( Iterator it = lParams.entrySet().iterator() ; it.hasNext() ; )
        //        {
        //            Map.Entry entrees = (Map.Entry) it.next();
        //            Object cle = entrees.getKey();
        //            Object valeur = entrees.getValue();
        //            if (lOv.containsKey(cle))
        //            {
        //                lOv.setFieldValue(cle, valeur);
        //            }
        //        }
        //
        //        _htData.put(sKey, sValue);
    }

    //    public void setFieldValue(String sKey, String sValue)
    //    {
    //        trace("setFieldValue(" + sKey + "," + sValue + ")");
    //        _htData.put(sKey, sValue);
    //    }

    /**
     * Returns a single field value for the given FORM NAME and FIELD NAME
     * @param sFormName
     * @param sFieldName
     * @return
     */
    public String getFieldValue(String sFormName, String sFieldName)
    {
        return getFieldValue(composeFieldKey(sFormName, sFieldName));
    }

    /**
     * Returns a single field value for the given HashTable key
     * @param sKey
     * @return
     */
    private String getFieldValue(String sKey)
    {
        trace("getFieldValue(" + sKey + ")");
        if (sKey == null)
        {
            return null;
        }
        Object obj = _htData.get(sKey);
        if (obj != null)
        {
            if (obj instanceof String)
            {
                return (String) obj;
            }
            else if (obj instanceof String[])
            {
                String[] values = (String[]) obj;
                if (values != null)
                {
                    if (values.length > 0)
                    {
                        return values[0];
                    }
                }
            }
            else
            {
                error("Object for key '" + sKey + "' is not a String or String[] instance");
                return null;
            }
        }
        return null;
    }

    /**
     * Returns multiple field values for the given FORM NAME and FIELD NAME
     * @param sFormName
     * @param sFieldName
     * @return
     */
    public String[] getFieldValues(String sFormName, String sFieldName)
    {
        return getFieldValues(composeFieldKey(sFormName, sFieldName));
    }
    
    /**
     * Returns multiple field values for the given HashTable key
     * @param sKey
     * @return
     */
    private String[] getFieldValues(String sKey)
    {
        trace("getFieldValues(" + sKey + ")");
        if (sKey == null)
        {
            return null;
        }
        Object obj = _htData.get(sKey);
        if (obj != null)
        {
            if (obj instanceof String)
            {
                String values[] = { (String) obj };
                return values;
            }
            else if (obj instanceof String[])
            {
                return (String[]) obj;
            }
            else
            {
                error("Object for key '" + sKey + "' is not a String or String[] instance");
                return null;
            }
        }
        return null;
    }

//    public void removeFieldValue(String sKey)
//    {
//        trace("removeFieldValue(" + sKey + ")");
//        if (sKey != null)
//        {
//            _htData.remove(sKey);
//        }
//    }

}