package org.objectweb.telosys.uil.screenmap;

import java.io.InputStream;
import java.util.Iterator;

import javax.servlet.ServletContext;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.Tool;
import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.uil.i18n.Translator;
import org.objectweb.telosys.util.StrUtil;
import org.objectweb.telosys.util.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * The screens configuration loader<br>
 * Used to load the screens definitions from the "screens.xml" file
 * 
 * @author Laurent GUERIN
 *  
 */
public class ScreenDefLoader extends TelosysObject
{
    //private final static String HTML = "html";
    //private final static String HTML               = ScreenMapConst.HTML_SCREEN_TYPE;

    //private final static String XUL = "xul";
    //private final static String XUL                = ScreenMapConst.XUL_SCREEN_TYPE;

    // private ScreenDefinitions   _screenDefinitions = null; // removed in v 1.0.5

    
    /**
     * 
     */
    public ScreenDefLoader()
    {
        super();
    }
    
    //---------------------------------------------------------------------------
    private Element toXmlElement(Node node)
    {
        if (node instanceof Element)
        {
            return (Element) node;
        }
        else
        {
            return null;
        }
    }

    //---------------------------------------------------------------------------
    // tag <xxx> <html ...> <xul ...> </xxx>
    //---------------------------------------------------------------------------
    private String getRootDir(Node tag, String sChildName)
    {
        trace("getRootDir : tag = " + tag.getNodeName() + "  child name = " + sChildName);
        NodeList childNodes = tag.getChildNodes(); // <html ...> or <xul ...>
        if (childNodes != null)
        {
            for ( int i = 0 ; i < childNodes.getLength() ; i++ )
            {
                Node node = childNodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE)
                {
                    Element elem = toXmlElement(node);
                    if (elem != null)
                    {
                        if (elem.getNodeName().equals(sChildName))
                        {
                            String sRet = elem.getAttribute("root");
                            trace(" -> " + sRet);
                            return sRet;
                        }
                    }
                }
            }
        }
        trace(" -> not found ! ");
        return null;
    }

    //---------------------------------------------------------------------------
    // tag <rootdir>
    //---------------------------------------------------------------------------
    private void processRootDir(ScreenDefinitions screenDefinitions, NodeList list)
    {
        trace("processRootDir()...");
        if (screenDefinitions == null)
        {
            error("processRootDir : screenDefinitions = null !");
            return;
        }
        if (list == null)
        {
            error("processRootDir : no node ( list = null ) !");
            return;
        }

        //--- Get the HTML root dir config
        ScreenDefRootDir htmlRootDir = screenDefinitions.getHtmlRootDir();
        if (htmlRootDir == null)
        {
            error("processRootDir : No HTML root dir instance in screen definitions !");
            return;
        }
        //--- Get the XUL root dir config
        ScreenDefRootDir xulRootDir = screenDefinitions.getXulRootDir();
        if (xulRootDir == null)
        {
            error("processRootDir : No XUL root dir instance in screen definitions !");
            return;
        }
        //--- for each "rootdir" node
        for ( int iRootDir = 0 ; iRootDir < list.getLength() ; iRootDir++ )
        {
            Node tagRootdir = list.item(iRootDir);
            //trace("processRootDir() : node name = " + tagRootdir.getNodeName() + " / i = " + iRootDir );
            NodeList childNodes = tagRootdir.getChildNodes();
            if (childNodes != null)
            {
                //trace("processRootDir() : childNodes.getLength() = " + childNodes.getLength() );
                //--- for each child : "template", "body", "page", ....
                for ( int iChild = 0 ; iChild < childNodes.getLength() ; iChild++ )
                {
                    Node node = childNodes.item(iChild);
                    //trace("processRootDir() : node.getNodeType() = " + node.getNodeType() );
                    if (node.getNodeType() == Node.ELEMENT_NODE)
                    {
                        trace("processRootDir() : <rootdir> <" + node.getNodeName() + "> ...");
                        if (node.getNodeName().equals("template"))
                        {
                            htmlRootDir.setTemplateRootDir(getRootDir(node, ScreenMapConst.HTML_SCREEN_TYPE));
                            xulRootDir.setTemplateRootDir(getRootDir(node, ScreenMapConst.XUL_SCREEN_TYPE));
                        }
                        else if (node.getNodeName().equals("body"))
                        {
                            htmlRootDir.setBodyRootDir(getRootDir(node, ScreenMapConst.HTML_SCREEN_TYPE));
                            xulRootDir.setBodyRootDir(getRootDir(node, ScreenMapConst.XUL_SCREEN_TYPE));
                        }
                        else if (node.getNodeName().equals("css"))
                        {
                            htmlRootDir.setCssRootDir(getRootDir(node, ScreenMapConst.HTML_SCREEN_TYPE));
                            xulRootDir.setCssRootDir(getRootDir(node, ScreenMapConst.XUL_SCREEN_TYPE));
                        }
                        else if (node.getNodeName().equals("images"))
                        {
                            htmlRootDir.setImagesRootDir(getRootDir(node, ScreenMapConst.HTML_SCREEN_TYPE));
                            xulRootDir.setImagesRootDir(getRootDir(node, ScreenMapConst.XUL_SCREEN_TYPE));
                        }
                        else if (node.getNodeName().equals("script"))
                        {
                            htmlRootDir.setScriptRootDir(getRootDir(node, ScreenMapConst.HTML_SCREEN_TYPE));
                            xulRootDir.setScriptRootDir(getRootDir(node, ScreenMapConst.XUL_SCREEN_TYPE));
                        }
                        else if (node.getNodeName().equals("page"))
                        {
                            htmlRootDir.setPageRootDir(getRootDir(node, ScreenMapConst.HTML_SCREEN_TYPE));
                            xulRootDir.setPageRootDir(getRootDir(node, ScreenMapConst.XUL_SCREEN_TYPE));
                        }
                        else
                        {
                            error("processRootDir : unknown tag <" + node.getNodeName() + ">");
                        }
                    }
                }
            }
            else
            {
                error("processRootDir : no childs for tag 'rootdir' (" + iRootDir + ") ");
            }
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Initialize the <screen_conventions> attributes if any
     * @param screenDefinitions
     * @param list
     * @since 1.0.5
     */
    private void processScreenConventions(ScreenConventions screenConventions, NodeList list)
    {
        trace("processScreenDefaults()...");
        if (list == null)
        {
            //error("processScreenDefaults : no node ( list = null ) !");
            // It's not an error : the <screen_conventions> tag is not mandatory
            return;
        }

        String sValue = null;
        //--- for each "screen_conventions" node
        for ( int i = 0 ; i < list.getLength() ; i++ )
        {
            Node node = list.item(i);
            Element element = toXmlElement(node);
            if (element != null)
            {
                sValue = element.getAttribute("body");
                if (isValid(sValue))
                {
                    screenConventions.setBody(sValue);
                }
                sValue = element.getAttribute("script");
                if (isValid(sValue))
                {
                    screenConventions.setScript(sValue);
                }
                sValue = element.getAttribute("css");
                if (isValid(sValue))
                {
                    screenConventions.setCss(sValue);
                }
                sValue = element.getAttribute("template");
                if (isValid(sValue))
                {
                    screenConventions.setTemplate(sValue);
                }
                sValue = element.getAttribute("contextid");
                if (isValid(sValue))
                {
                    screenConventions.setContextId(sValue);
                }
                sValue = element.getAttribute("contextname");
                if (isValid(sValue))
                {
                    screenConventions.setContextName(sValue);
                }
                sValue = element.getAttribute("contextaction");
                if (isValid(sValue))
                {
                    screenConventions.setContextAction(sValue);
                }            
            }
        }
    }

    //---------------------------------------------------------------------------
    // tag <screen>
    //---------------------------------------------------------------------------
    /**
     * Load all the given "screen" XML nodes
     * @param screenDefinitions
     * @param list all the "screen" nodes to process
     */
    private void processScreen(ScreenDefinitions screenDefinitions, NodeList list)
    {
        trace("processScreen()...");

        if (screenDefinitions == null)
        {
            error("processScreen : screenDefinitions = null !");
            return;
        }
        if (list == null)
        {
            error("processScreen : no node ( list = null ) !");
            return;
        }

        ScreenDefinition screen = null;
        String sScreenName = null;
        String sScreenBody = null;
        String sScreenTemplate = null;

        for ( int i = 0 ; i < list.getLength() ; i++ ) // for each <screen> tag
        {
            //--- New screen
            Element screenTag = toXmlElement(list.item(i));
            if (screenTag != null)
            {
                //--- screen attributes
                // NB .getAttribute() returns "" if the attribute is not found (doesn't return null)
                sScreenName = screenTag.getAttribute("name");
                //if (sScreenName != null)
                if ( ! StrUtil.nullOrVoid(sScreenName) ) // v 1.0.5
                {
                    //--- New instance
                    ScreenDefLocation screenLoc = new ScreenDefLocation();
                    screenLoc.setScreenBody(screenTag.getAttribute("body"));
                    screenLoc.setScreenTemplate(screenTag.getAttribute("template"));
                    screenLoc.setScreenPage(screenTag.getAttribute("page"));
                    screenLoc.setScreenUrl(screenTag.getAttribute("url"));
                    screenLoc.setScreenCss(screenTag.getAttribute("css"));
                    screenLoc.setScreenScript(screenTag.getAttribute("script"));

                    String sContextId     = screenTag.getAttribute("contextid"); // v 1.0.5
                    String sContextName   = screenTag.getAttribute("contextname"); // v 1.0.5
                    String sContextAction = screenTag.getAttribute("contextaction"); // v 1.0.5
                    
                    //screen = new ScreenDefinition(sScreenName, screenLoc); // v 1.0.5
                    screen = new ScreenDefinition(sScreenName, screenLoc, sContextId, sContextName, sContextAction);// v 1.0.5

                    //--- screen childs : <html ..> <xul ..> <element ..>
                    NodeList childNodes = screenTag.getChildNodes();
                    if (childNodes != null)
                    {
                        for ( int iChild = 0 ; iChild < childNodes.getLength() ; iChild++ )
                        {
                            Node node = childNodes.item(iChild);
                            if (node.getNodeType() == Node.ELEMENT_NODE)
                            {
                                Element tag = toXmlElement(node);
                                String sNodeName = node.getNodeName();
                                if ((sNodeName.equals(ScreenMapConst.HTML_SCREEN_TYPE)) || (sNodeName.equals(ScreenMapConst.XUL_SCREEN_TYPE)))
                                {
                                    //--- Tags <html ...> or <xul ...>
                                    trace("new ScreenDefLocation(" + sNodeName + ")");
                                    ScreenDefLocation loc = new ScreenDefLocation(sNodeName);
                                    loc.setScreenBody(tag.getAttribute("body"));
                                    loc.setScreenTemplate(tag.getAttribute("template"));
                                    loc.setScreenPage(tag.getAttribute("page"));
                                    loc.setScreenUrl(tag.getAttribute("url"));
                                    loc.setScreenCss(tag.getAttribute("css"));
                                    loc.setScreenScript(tag.getAttribute("script"));
                                    screen.setScreenDefLocation(loc);
                                }
                                else if (sNodeName.equals("element"))
                                {
                                    //--- Tag <element ...>
                                    String sName = tag.getAttribute("name");
                                    String sValue = tag.getAttribute("value");
                                    String sContent = tag.getAttribute("content");
                                    trace("new ScreenMapElement(" + sName + "," + sValue + "," + sContent + ")");
                                    screen.addElement(new ScreenMapElement(sName, sValue, sContent));
                                }
                            }
                        } // for each child node
                    } // if (childNodes != null)
                    //---
                    trace("addScreenDefinition : name " + screen.getScreenName() + " - " + screen.getElemCount());
                    screenDefinitions.addScreenDefinition(screen);
                }
                else
                {
                    error("Null or void screen name detected ! ");
                }
            }
        }
    }

    //---------------------------------------------------------------------------
    // tag <config>
    //---------------------------------------------------------------------------
    private boolean isValid(String sValue)
    {
        if (sValue != null)
        {
            if (sValue.trim().length() > 0)
            {
                return true;
            }
        }
        return false;
    }

    //---------------------------------------------------------------------------
    private void processConfig(ScreenDefConfig config, NodeList list)
    {
        trace("processConfig()...");
        if (list == null)
        {
            error("processConfig : no node ( list = null ) !");
            return;
        }

        String sValue = null;
        //--- for each "config" node ( ONLY ONE EXPECTED )
        for ( int i = 0 ; i < list.getLength() ; i++ )
        {
            Node node = list.item(i);
            Element element = toXmlElement(node);
            if (element != null)
            {
                sValue = element.getAttribute("default_action");
                if (isValid(sValue))
                {
                    config.setDefaultAction(sValue);
                }
                sValue = element.getAttribute("telosys_jar_file");
                if (isValid(sValue))
                {
                    config.setTelosysJarFile(sValue);
                }
                sValue = element.getAttribute("framework_javascript_file");
                if (isValid(sValue))
                {
                    config.setFrameworkJavascriptFile(sValue);
                }
                sValue = element.getAttribute("framework_css_dir");
                if (isValid(sValue))
                {
                    config.setFrameworkCSSDir(sValue);
                }
                sValue = element.getAttribute("screen_map_url");
                if (isValid(sValue))
                {
                    config.setScreenMapUrl(sValue);
                }
                sValue = element.getAttribute("ajax_request_url");
                if (isValid(sValue))
                {
                    config.setAjaxRequestUrl(sValue);
                }
                sValue = element.getAttribute("base_language");
                if (isValid(sValue))
                {
                    config.setBaseLanguage(sValue);
                }
                sValue = element.getAttribute("all_languages");
                if (isValid(sValue))
                {
                    config.setAllLanguages(sValue);
                }
            }
        }
    }

    //----------------------------------------------------------------------------------
    /**
     * Loads the screens definitions from the given XML file
     * 
     * @param sFileName :
     *            the XML file to load ( e.g. "screens.xml" )
     * @return
     */
//    public ScreenDefinitions load(String sFileName)
//    {
//        trace("load(" + sFileName + ")...");
//
//        //--- The screens definitions container
//        ScreenDefinitions screenDefinitions = null;
//
//        //--- Initialize the screens definitions from the XML file
//        Document document = XmlUtil.loadFile(sFileName);
//        if (document != null)
//        {
//            //--- Create the screens definitions container
//            screenDefinitions = new ScreenDefinitions();
//
//            //--- Process the tag <config> ( only one tag expected )
//            NodeList configNodes = document.getElementsByTagName("config");
//            processConfig(screenDefinitions, configNodes);
//
//            //--- Process the tags <rootdir> ( only one tag expected )
//            NodeList rootdirNodes = document.getElementsByTagName("rootdir");
//            processRootDir(screenDefinitions, rootdirNodes);
//
//            //--- Process the tags <screen> ( N tags )
//            NodeList screenNodes = document.getElementsByTagName("screen");
//            processScreen(screenDefinitions, screenNodes);
//
//        }
//        _screenDefinitions = screenDefinitions;
//        return _screenDefinitions;
//    }
    //----------------------------------------------------------------------------------
// removed in v 1.0.5
//    public ScreenDefinitions load(String sFileName)
//    {
//        trace("load('" + sFileName + "')...");
//        if ( sFileName == null )
//        {
//            error("Cannot load ScreenDefinitions : File name is null " );
//        }
//        Document document = null ;
//        try
//        {
//            document = XmlUtil.parse(sFileName);
//        } catch (TelosysException e)
//        {
//            error("Cannot load ScreenDefinitions from file '"+sFileName+"' : " + e.getMessage() );
//            return null ;
//        }
//        return load(document);
//    }
    //----------------------------------------------------------------------------------
    //public ScreenDefinitions load(InputStream is, ServletContext servletContext)
    /**
     * Loads the screens definitions and the screens translations if any <br>
     * 
     * @param is
     * @param servletContext
     * @return screens definitions instance, or null if they cannot be loaded
     */
    public ScreenDefinitions loadDefinitionsAndTranslations(InputStream is, ServletContext servletContext)
    {
        trace("load(InputStream)...");
        if ( is == null )
        {
            error("Cannot load ScreenDefinitions : InputStream is null " );
        }
        Document document = null ;
        try
        {
            document = XmlUtil.parse(is);
        } catch (TelosysException e)
        {
            error("Cannot load ScreenDefinitions from InputStream : " + e.getMessage() );
            return null ;
        }
        
        //--- 1) Load the screens definitions
        Telosys.info("Load screens definitions ... "  );        
        ScreenDefinitions screenDefinitions = loadDefinitions(document, servletContext);
        
        if ( screenDefinitions != null )
        {
            //--- 2) Load the screens translations (if any)
            Telosys.info("Load screens translations ... "  );        
            loadTranslations(screenDefinitions, servletContext); // v 1.0.5
        }
        else
        {
            Telosys.error("Cannot load screens definitions ! "  );        
        }
        
        return screenDefinitions ;
    }
    //----------------------------------------------------------------------------------
    private ScreenDefinitions loadDefinitions(Document document, ServletContext servletContext)
    {
        //--- The screens definitions container
        if (document != null)
        {
            //--- Create the screens definitions container
            ScreenDefinitions screenDefinitions = new ScreenDefinitions();

            //--- Process the tag <config> ( only one tag expected )
            ScreenDefConfig config = screenDefinitions.getConfig() ;
            processConfig(config, document.getElementsByTagName("config") );
            
            //--- Process the tags <rootdir> ( only one tag expected )
            NodeList rootdirNodes = document.getElementsByTagName("rootdir");
            processRootDir(screenDefinitions, rootdirNodes);

            //--- Process the tags <screen_conventions> ( only one tag expected ) // v 1.0.5
            ScreenConventions screenConventions = screenDefinitions.getScreenConventions() ;
            processScreenConventions(screenConventions, document.getElementsByTagName("screen_conventions") );

            //--- Process the tags <screen> ( N tags )
            NodeList screenNodes = document.getElementsByTagName("screen");
            processScreen(screenDefinitions, screenNodes);

            //--- Keep the Screen Definitions
            // _screenDefinitions = screenDefinitions; // removed in v 1.0.5
            return screenDefinitions ; // v 1.0.5
        }
        else
        {
            // _screenDefinitions = null ; // removed in v 1.0.5
            error("Cannot load ScreenDefinitions from Document : Document is null !" );
            return null ; // v 1.0.5
        }
        //return _screenDefinitions;
    }
    //----------------------------------------------------------------------------------
    //private String getFilePath(String sFileArg, String sType)
    private String getFilePath(ScreenDefinitions screenDefinitions, String sFileArg, String sType) // v 1.0.5
    {
        if (sFileArg == null)
        {
            return null;
        }
        String sFile = sFileArg.trim();

        //if (_screenDefinitions != null)
        if ( screenDefinitions != null ) // v 1.0.5
        {
            // ScreenDefRootDir rootDir = _screenDefinitions.getRootDir(sType);
            ScreenDefRootDir rootDir = screenDefinitions.getRootDir(sType); // v 1.0.5
            if (rootDir != null)
            {
                String sRootDir = rootDir.getBodyRootDir();
                if (sRootDir != null)
                {
                    return Tool.buildPath(sRootDir, sFile, "/");
                }
                else
                {
                    return sFile; // No root dir => AS IS
                }
            }
        }
        return null;
    }

    /**
     * Loads the translations files (if any)
     * @param servletContext
     */
    // public void loadTranslations(ServletContext servletContext)
    private void loadTranslations(ScreenDefinitions screenDefinitions, ServletContext servletContext)
    {
        trace("loadTranslations(screenDefinitions, ServletContext)");
        //if (_screenDefinitions == null)
        if ( screenDefinitions == null ) // v 1.0.5
        {
            error("loadTranslations() : _screenDefinitions is null ");
            return;
        }

        //--- Languages list 
        //ScreenDefConfig config = _screenDefinitions.getConfig();
        ScreenDefConfig config = screenDefinitions.getConfig(); // v 1.0.5
        String[] languages = config.getAllLanguagesArray();

        info("Translation languages : [" + config.getBaseLanguage() + "]," + config.getAllLanguages() );

        if ( languages == null )
        {
            info("No languages defined => no translation to load.");
            return;
        }
        if ( languages.length == 0 )
        {
            info("Zero languages defined => no translation to load.");
            return;
        }
        
        //--- Try to load translations for each screen
        //        _screenDefinitions.getRootDir(ScreenMapConst.HTML_SCREEN_TYPE);
        info("Loading translation files ( languages : " + config.getAllLanguages()+" )");
        // Iterator iter = _screenDefinitions.getScreenNames();
        Iterator iter = screenDefinitions.getScreenNames(); // v 1.0.5
        while (iter.hasNext())
        {
            String sScreenName = (String) iter.next();
            trace("----- loading translations for screen " + sScreenName);
            if (sScreenName != null)
            {
                // ScreenDefinition screenDef = _screenDefinitions.getScreenDefinition(sScreenName);
                ScreenDefinition screenDef = screenDefinitions.getScreenDefinition(sScreenName); // v 1.0.5
                String sScreenBodyFile = screenDef.getScreenBody(ScreenMapConst.HTML_SCREEN_TYPE);
                //String sFile = getFilePath(sScreenBodyFile, ScreenMapConst.HTML_SCREEN_TYPE);
                String sFile = getFilePath(screenDefinitions, sScreenBodyFile, ScreenMapConst.HTML_SCREEN_TYPE); // v 1.0.5
                trace("loading translations for screen " + sScreenName + " : screen body = " + sScreenBodyFile + " -> " + sFile);

                //--- Try to load a translator for this Screen Map
                //Translator translator = FileTranslatorLoader.loadTranslations(servletContext, sFile, languages );
//                ITranslatorLoader translatorLoader = TelosysUIL.getTranslatorLoader();
//                Translator translator = null;
//                if (translatorLoader != null) translator = translatorLoader.loadTranslations(servletContext, sFile, languages );
                Translator translator = TelosysUIL.getTranslator(servletContext, sFile, languages); // v 1.0.3
                
                if ( translator != null )
                {
                    //--- Store the translator in the Screen Definition
                    screenDef.setTranslator(translator);
                }
            }
            else
            {
                error("loadTranslations() : Screen Name is null !");
            }
        }

        /***********************************************************************************************************************************************************************************************
         * 
         * //--- Get all languages defines for the current application String[] languages = { "fr", "en" }; // TODO // String sTransDir = "/screen/" ; // TODO // ScreenDefConfig config =
         * _screenDefinitions.getConfig(); // ScreenDefRootDir htmlRootDir = getRootDir(ScreenMapConst.HTML_SCREEN_TYPE); // ScreenDefRootDir htmlRootDir = getRootDir(ScreenMapConst.HTML_SCREEN_TYPE); //
         * ScreenDefRootDir xulRootDir = _screenDefinitions.getXulRootDir();
         * 
         * String sHtmlBodyRootDir = getRootDir(ScreenMapConst.HTML_SCREEN_TYPE); String sXulBodyRootDir = getRootDir(ScreenMapConst.XUL_SCREEN_TYPE); if ( sHtmlBodyRootDir != null || sXulBodyRootDir !=
         * null ) { String sTransDir = null; //"/screen/" ; ScreenMapTranslator translator = new ScreenMapTranslator(); if ( sHtmlBodyRootDir != null ) { sTransDir = sHtmlBodyRootDir ; //--- Try to
         * load a translation file for each language for ( int i=0 ; i < languages.length ; i++ ) { String sLang = languages[i] ; if ( sLang != null ) { String sFileName = sScreenName + "_" + sLang +
         * ".txt" ; trace(" . loading translation file " + sFileName ); InputStream is = servletContext.getResourceAsStream(sTransDir+sFileName); } } loadTranslationsFile( servletContext, translator ); } }
         **********************************************************************************************************************************************************************************************/
    } //    public void loadTranslationsFile( ServletContext servletContext, ScreenMapTranslator translator )

//    private Translator loadScreenTranslations(ServletContext servletContext, String sScreenBodyFile, String[] languages )
//    {
//        if ( sScreenBodyFile != null )
//        {            
//	        if ( sScreenBodyFile.endsWith(".jsp") )
//	        {
//	            String s = sScreenBodyFile.substring(0,sScreenBodyFile.length()-4);
//	            String sFile = null ;
//	            if ( languages != null )
//	            {
//	                ScreenMapTranslator translator = new ScreenMapTranslator();
//	                for ( int i=0 ; i < languages.length ; i++ )
//	                {
//	                    String sLang = languages[i];
//	                    sFile = s + "_" + sLang + ".txt" ;
//	                    loadTranslationsFile( servletContext, sFile, translator, sLang );
//	                }
//	                return translator ;
//	            }
//	        }
//	        else
//	        {
//	            error("ScreenBody File doesn't end with '.jsp'");
//	        }
//        }
//        else
//        {
//            error("ScreenBody File is null");
//        }
//        return null;
//    }    
    
//    private void loadTranslationsFile(ServletContext servletContext, String sFile, Translator translator, String sLang )
//    {
//        trace("loadTranslationsFile(ctx, file='" + sFile + "', language='" + sLang + "')");
//        InputStream inputStream = servletContext.getResourceAsStream(sFile) ;
//        if ( inputStream != null )
//        {
//            trace("+++ loading " + sFile );
////            ScreenMapTranslator translator = new ScreenMapTranslator();
//            translator.load(inputStream, sLang);
//        }
//        else
//        {
//            trace("File " + sFile + " not found ( input stream is null ) ");
//        }
//        
//    }
}