package org.objectweb.telosys.uil.screenmap;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * This class holds the "root directories" loaded from the "screens.xml" file <br>
 * There are 2 instances of this class (one for "html", one for "xul" ) held by ScreenDefinitions <br>
 * 
 * @author Laurent GUERIN
 * 
 */
public class ScreenDefRootDir extends TelosysObject
{
    private String   _sScreenType      = null ;
    
    // <rootdir><template>
    private String   _sTemplateRootDir = "/screen/template";

    // <rootdir><body>
    private String   _sBodyRootDir     = "/screen";

    // <rootdir><css>
    private String   _sCssRootDir      = "/screen/css";

    // <rootdir><images>
    private String   _sImagesRootDir   = "/screen/images";

    // <rootdir><script>
    private String   _sScriptRootDir   = "/screen/script";

    // <rootdir><page>
    private String   _sPageRootDir     = "/page";

    //---------------------------------------------------------------------------
    /**
     * Constructor
     * @since 1.0.5
     */
    public ScreenDefRootDir( String sScreenType )
    {
        super();
        if ( sScreenType != null )
        {
            if ( sScreenType.equals( ScreenMapConst.HTML_SCREEN_TYPE ) || sScreenType.equals( ScreenMapConst.XUL_SCREEN_TYPE ) )
            {
                _sScreenType = sScreenType ;
            }
            else
            {
                throw new TelosysRuntimeException("Invalid screen type '" + sScreenType + "'");
            }
        }
        else
        {
            throw new TelosysRuntimeException("Screen type is null !");
        }
    }

    
    //---------------------------------------------------------------------------
    private String buildURL(String sFile, String sRootDir)
    {
        if (sFile != null)
        {
            if (sFile.indexOf('/') < 0) // No "/" in the File Name
            {
                return sRootDir + "/" + sFile;
            }
            else
            {
                return sFile;
            }
        }
        else
        {
            return null;
        }
    }
    //---------------------------------------------------------------------------
    // SCREEN TYPE
    //---------------------------------------------------------------------------
    /**
     * Returns the screen type for this root dir definition
     * @return "html" or "xul"
     */
    public String getScreenType()
    {
        return _sScreenType ;
    }
    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // TEMPLATE
    //---------------------------------------------------------------------------
    /* package */ void setTemplateRootDir(String s)
    {
        _sTemplateRootDir = s ;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "root directory" for an "template resource"
     * @return e.g. "/screen/template"
     */
    public String getTemplateRootDir()
    {
        return _sTemplateRootDir ;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "full path" for the given "template resource"
     * @param sFile
     * @return
     */
    public String getTemplatePath(String sFile)
    {
        return buildURL(sFile, _sTemplateRootDir);
    }

    //---------------------------------------------------------------------------
    // SCREEN BODY
    //---------------------------------------------------------------------------
    /* package */ void setBodyRootDir(String s)
    {
        _sBodyRootDir = s;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "root directory" for a "screen body"
     * @return e.g. "/screen"
     */
    public String getBodyRootDir()
    {
        return _sBodyRootDir;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "full path" for the given "screen body"
     * @param sFile
     * @return
     */
    public String getBodyPath(String sFile)
    {
        return buildURL(sFile, _sBodyRootDir);
    }

    //---------------------------------------------------------------------------
    // PAGE
    //---------------------------------------------------------------------------
    /* package */ void setPageRootDir(String s)
    {
        _sPageRootDir = s;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "root directory" for an "page resource"
     * @return
     */
    public String getPageRootDir()
    {
        return _sPageRootDir;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "full path" for the given "page resource"
     * @param sFile
     * @return
     */
    public String getPagePath(String sFile)
    {
        return buildURL(sFile, _sPageRootDir);
    }

    //---------------------------------------------------------------------------
    // CSS
    //---------------------------------------------------------------------------
    /* package */ void setCssRootDir(String s)
    {
        _sCssRootDir = s ;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "root directory" for a "CSS resource"
     * @return
     */
    public String getCssRootDir()
    {
        return _sCssRootDir;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "full path" for the given "CSS resource"
     * @param sFile
     * @return
     */
    public String getCssPath(String sFile)
    {
        return buildURL(sFile, _sCssRootDir);
    }

    //---------------------------------------------------------------------------
    // IMAGES
    //---------------------------------------------------------------------------
    /* package */ void setImagesRootDir(String s)
    {
        _sImagesRootDir = s;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "root directory" for an "image resource"
     * @return
     */
    public String getImagesRootDir()
    {
        return _sImagesRootDir;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "full path" for the given "image resource"
     * @param sFile
     * @return
     */
    public String getImagePath(String sFile)
    {
        return buildURL(sFile, _sImagesRootDir);
    }

    //---------------------------------------------------------------------------
    // SCRIPT
    //---------------------------------------------------------------------------
    /* package */ void setScriptRootDir(String s)
    {
        _sScriptRootDir = s;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "root directory" for an "script resource" ( javascript )
     * @return
     */
    public String getScriptRootDir()
    {
        return _sScriptRootDir;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "full path" for the given "script resource" ( javascript )
     * @param sFile
     * @return
     */
    public String getScriptPath(String sFile)
    {
        return buildURL(sFile, _sScriptRootDir);
    }

    //---------------------------------------------------------------------------

}