package org.objectweb.telosys.uil.screenmap;

import org.objectweb.telosys.common.TelosysObject;

/**
 * Element designed to be used in a screen template <br>
 * One or more elements can be defined for each screenmap ( see "element" tag in "screens.xml" )
 * 
 * @author Laurent GUERIN
 *
 */
public class ScreenMapElement extends TelosysObject
{
    private String  _sName = null ;

    private String  _sValue = null ;

    private String  _sContent = null ;

    /**
     * Constructor
     * @param sName
     * @param sValue
     * @param sContent
     */
    public ScreenMapElement(String sName, String sValue, String sContent)
    {
        super();
        _sName = sName ;
        _sValue = sValue;
        _sContent = sContent;
    }
    
    /**
     * Returns the element name
     * @return 
     */
    public String getName()
    {
        return _sName;
    }

    /**
     * Returns the element value
     * @return
     */
    public String getValue()
    {
        return _sValue;
    }
    
    /**
     * Returns the element content
     * @return
     */
    public String getContent()
    {
        return _sContent;
    }
    
    /**
     * Returns true if the element is a value
     * @return
     */
    public boolean isValue()
    {
        if ( _sValue == null ) return false ;
        if ( _sValue.trim().length() == 0 ) return false ;
        return true ;
    }
    
    /**
     * Returns true if the element is a content to include in the output stream
     * @return
     */
    public boolean isContent()
    {
        if ( _sContent == null ) return false ;
        if ( _sContent.trim().length() == 0 ) return false ;
        return true ;
    }
    
}
