package org.objectweb.telosys.upload;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysProperties;
import org.objectweb.telosys.common.TelosysServlet;
import org.objectweb.telosys.util.StrUtil;

/**
 * Standard servlet for single file download 
 * 
 * @author Laurent GUERIN
 *
 */
public class FileDownloadServlet extends TelosysServlet // extends HttpServlet
{
    //----------------------------------------------------------------------------------
    // CONSTANTS
    //----------------------------------------------------------------------------------
    private static final String RESPONSE_CONTENT_TYPE          = "text/html; charset=UTF-8";

    //--- Request parameters ( set by the tag "fileupload" )
    private static final String PARAM_DEST_DIR                 = "dest_dir";

    private static final String PARAM_FILE_NAME                = "file_name";

    private static final String PARAM_CONTEXT_ID               = "context_id";

    private static final String PARAM_CONTEXT_ATTRIBUTE        = "context_attribute";

    private static final String PARAM_SESSION_KEY              = "session_key";

    //----------------------------------------------------------------------------------

    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    protected void doGet(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        trace("doGet()...");
        process(httpRequest, httpResponse);
    }

    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    protected void doPost(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        trace("doPost()...");
        process(httpRequest, httpResponse);
    }

    /**
     * Not void
     */
    private boolean notVoid(String s)
    {
        return (s != null && s.length() > 0);
    }
    
    /**
     * Standard servlet processing 
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    private void process(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        ServletOutputStream out = response.getOutputStream(); // Cannot be called more than 1 time !!

        //String sDir = "";
        
        //----------------------------------------------------------------------------------
        if ( notVoid(request.getParameter(PARAM_FILE_NAME)) ) 
        {
            //--- Get file content from the FILE SYSTEM            
            File file = null ;
            String sUploadDir = null ;
            Properties prop = Telosys.getProperties();
            if ( prop != null )
            {
                sUploadDir = prop.getProperty(TelosysProperties.UPLOAD_DIR);
            }
            if ( sUploadDir != null )
            {
	            String sFileName = (String) request.getParameter(PARAM_FILE_NAME);
	            String sDir = (String) request.getParameter(PARAM_DEST_DIR);
	            //--- Get file content from FILE SYSTEM ( content from file on disk )	            
	            try
                {
                    file = FileManager.getFileFromFilesystem(sUploadDir, sDir, sFileName);
                } catch (TelosysException e)
                {
                    file = null ;
                    String sErrorMsg = "<h2>Cannot get file from file system !</h2>" 
            	        + "<h2> " + e.getMessage() + " </h2>"
            	        + "<h3>Upload dir = " + sUploadDir + " </h3>"
            	        + "<h3>Directory parameter = '" + sDir + "' </h3>"
            	        + "<h3>File name parameter = '" + sFileName + "' </h3>"
            	        ;
                    generateErrorResponse( response, out, sErrorMsg );
                }
            }
            else
            {
                file = null ;
                String sErrorMsg = "<h2>Cannot get file from file system !</h2>" 
        	        + "<h2> The upload directory is not defined </h2>"
        	        ;
                generateErrorResponse( response, out, sErrorMsg );
            }
            
            //--- The file has been found => print the content
            if ( file != null )
            {
	            generateFileContentResponse( response, out, file );
            }
        }

        //----------------------------------------------------------------------------------
        else if (notVoid(request.getParameter(PARAM_CONTEXT_ID)) && notVoid(request.getParameter(PARAM_CONTEXT_ATTRIBUTE)))
        {
            int iContextId = -1 ;
            String sContextAttribute = "" ;            
            //--- Get file content from SCREEN CONTEXT ATTRIBUTE ( content in memory )
            iContextId = StrUtil.getInt(request.getParameter(PARAM_CONTEXT_ID), 0) ;
            sContextAttribute = request.getParameter(PARAM_CONTEXT_ATTRIBUTE) ;            
            FileUploaded file = FileManager.getFileFromScreenContext(request, iContextId, sContextAttribute );
            if ( file != null )
            {
                boolean bRet = generateFileContentResponse( response, out, file );
                if ( bRet != true )
                {
                    generateErrorResponse( response, out, "<h2>The requested file is void.</h2>" );
                }
            }
            else
            {
                String sErrorMsg = "<h2>Cannot get file from Screen Context !</h2>" 
        	        + "<h2>Context id = " + iContextId + " </h2>"
        	        + "<h2>Context attribute = '" + sContextAttribute + "' </h2>";
                generateErrorResponse( response, out, sErrorMsg );
            }
        }

        //----------------------------------------------------------------------------------
        else if (notVoid(request.getParameter(PARAM_SESSION_KEY)))
        {
            String sSessionKey = "" ;
            //--- Get file content from HTTP SESSION ATTRIBUTE ( content in memory )
            sSessionKey = request.getParameter(PARAM_SESSION_KEY) ;
            FileUploaded file = FileManager.getFileFromHttpSession(request, sSessionKey );
            if ( file != null )
            {
                boolean bRet = generateFileContentResponse( response, out, file );
                if ( bRet != true )
                {
                    generateErrorResponse( response, out, "<h2>The requested file is void.</h2>" );
                }
            }
            else
            {
                String sErrorMsg = "<h2>Cannot get file from Http Session !</h2>" 
        	        + "<h2>Session key = " + sSessionKey + "</h2>";
                generateErrorResponse( response, out, sErrorMsg );
            }
        }
        //----------------------------------------------------------------------------------
        else
        {
            String sErrorMsg = "<h2>Invalid parameter(s)</h2>" 
    	        + "<h2>Parameters : " + request.getQueryString() + " </h2>"
    	        + "<h2>Don't know where to get the file</h2>";
            generateErrorResponse( response, out, sErrorMsg );
        }
        
        out.flush();
        out.close();
    }


    private boolean generateFileContentResponse(HttpServletResponse response, ServletOutputStream out, FileUploaded file ) throws IOException
    {
        byte[] data = file.getData() ;
        if ( data == null ) return false ;
        
        //----- Response headers --------------------------------------

        //--- "Content-Type" ( Mime Type )
        String sType = file.getFileType() ;
        response.setContentType(sType);
		
        //--- "Content-Length" 
        response.setContentLength(data.length);
        
        //----- Response body ------------------------------------------
        out.write(data);
        
        return true ;
    }

    private boolean generateFileContentResponse(HttpServletResponse response, ServletOutputStream out, File file ) throws IOException
    {
        if (file.exists() != true ) return false ;
        if ( file.length() <= 0 ) return false ;
        
        //----- Response headers --------------------------------------
        
        //--- "Content-Type" ( Mime Type )
        String sMimeType = getMimeType(file);
		if (sMimeType != null) {
	        response.setContentType(sMimeType);
		}		

		//--- "Content-Length" 
        response.setContentLength((int)file.length());
        
        //--- "Last-Modified"
        // response.setDateHeader("Last-Modified", file.lastModified());

        //----- Response body ------------------------------------------
		BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file));
		BufferedOutputStream bos = new BufferedOutputStream(out);
		byte[] buffer = new byte[1024];
		boolean eof = false;
		while (!eof) {
			int length = bis.read(buffer);
			if (length == -1) {
				eof = true;
			} else {
				bos.write(buffer, 0, length);
			}
		}
		bis.close();
		bos.flush();
		bos.close();
		return true ;
    }

    private boolean generateErrorResponse(HttpServletResponse response, ServletOutputStream out, String msg ) throws IOException
    {
        response.setContentType( RESPONSE_CONTENT_TYPE );
        out.println("");
        out.println("<html>");
        out.println("<head>");
        out.println("<title>File download error</title>");
        out.println("</head> ");
        out.println("<body>");
        out.println("<h1>Cannot download the requested file !</h1>" );
        out.println(msg);
        out.println("</body>");
        out.println("</html>");
        return true ;
    }

    private String getMimeType(File file ) throws IOException
    {
        // Mime-types are detected based on your server settings.
        // You can specify aditional mime-types in your application's web.xml 
        ServletContext sc = getServletContext();
        String mimeType = sc.getMimeType(file.getName());
        
		return mimeType ;
    }
}