package org.objectweb.telosys.upload;


/**
 * A part created from a "multi-part" http request ( Content-Type: multipart/form-data ) <br>
 * A part can be an uploaded file or a simple parameter 
 * 
 */
/* package */ class Part
{

//    /** Default encoding */
//    private static final String ENCODING = "ISO-8859-1";

    //--- name of the form element 
    private String              _sName = null ;

    //--- content type of the part (param or file) 
    private String              _sContentType  = null ; // text/plain, ...

    //--- "file system" name of the file 
    private String              _sFileName = null;

    //--- contents of the file 
    private byte[]              _fileContent = null;

    //--- value of the form element ( if it's a parameter ) 
    private String              _sParameterValue  = null;
    
    private boolean             _isFile = false ;

//    /**
//     * Constructor
//     */
//    Part(String name, ServletInputStream in, String boundary, String contentType, String fileName) throws IOException
//    {
//        this._sName = name;
//        this._sFileName = fileName;
//        this._sContentType = contentType;
//        
//        
//        // Fill value
//        PartInputStream pis = new PartInputStream(in, boundary);
//        ByteArrayOutputStream baos = new ByteArrayOutputStream(512);
//        byte[] buf = new byte[128];
//        int read;
//        while ((read = pis.read(buf)) != -1)
//        {
//            baos.write(buf, 0, read);
//        }
//        pis.close();
//        baos.close();
//        this._value = baos.toByteArray();
//    }
    
    /**
     * Constructor for a "file" part
     * @param name
     * @param contentType
     * @param fileName
     * @param value
     */
    Part(String name, String contentType, String fileName, byte[] value ) //throws IOException
    {
        this._sName = name;
        this._sContentType = contentType;
        this._sFileName = fileName;
        this._fileContent = value;
        this._isFile = true ;
    }
    
    /**
     * Constructor for a "parameter" part
     * @param name
     * @param value
     */
    Part(String name, String value ) //throws IOException
    {
        this._sName = name;
        this._sParameterValue = value ;
        this._isFile = false ;
    }
    
    /**
     * Returns the name of the form element ( input name attribute in the HTML form )
     * @return
     */
    public String getName()
    {
        return _sName;
    }

    /**
     * Returns the content type of the file data (or null if the part is a parameter)
     * @return content type of the file data.
     */
    public String getFileContentType()
    {
        return _sContentType;
    }

    /**
     * Returns the original name of the file (or null if the part is a parameter)
     * @return
     */
    public String getFileName()
    {
        return _sFileName;
    }

    /**
     * Returns the content of the uploaded file (or null if the part is a parameter)
     * @return
     */
    public byte[] getFileContent()
    {
        return _fileContent;
    }

//    /**
//     * Returns the value of the parameter in as a string
//     */
//    public String getStringValue() throws UnsupportedEncodingException
//    {
//        return getStringValue(ENCODING);
//    }
//
//    /**
//     * Returns the value of the parameter in as a string
//     */
//    public String getStringValue(String encoding) throws UnsupportedEncodingException
//    {
//        return new String(_fileContent, encoding);
//    }

    /**
     * Returns the value of the parameter's value (or null if this part is a file) 
     * @return
     */
    public String getParameterValue()
    {
        return _sParameterValue ;
    }
    
    /**
     * Returns true if this part is a file (else it's a parameter)
     * @return
     */
    public boolean isFile()
    {
        // return (this._sFileName != null);
        return _isFile ;
    }
}