package org.objectweb.telosys.upload;

/**
 * 
 * All the http parameters provided by an upload request 
 * ( parameters set by the widget ) 
 * 
 * @author Laurent GUERIN
 * 
 *  
 */
public class UploadParameters
{

    //--- Limited data size
    private long   _lMaxSize          = -1;

    //--- Parameters for FileSystem storage
    private String _sDestDir          = null;

    //--- Parameters for ScreenContext storage
    private int    _iContextId        = -1;

    private String _sContextAttribute = null;

    //--- Parameters for Session storage
    private String _sSessionKey       = null;

    //----------------------------------------------------------------------------
    /**
     * Constructor
     */
    public UploadParameters()
    {
        super();
    }

    //----------------------------------------------------------------------------
    /**
     * Returns the screen context id where to store the file as an attribute
     * @return the screen context id ( or -1 if not set )
     */
    public int getContextId()
    {
        return _iContextId;
    }
    /**
     * Set the screen context id where to store the file as an attribute
     * @param paramContextId
     */
    public void setContextId(int paramContextId)
    {
        _iContextId = paramContextId;
    }
    
    //----------------------------------------------------------------------------
    /**
     * Returns the maximum size of the uploaded file
     * @return the maximum size ( or -1 if not set )
     */
    public long getMaxSize()
    {
        return _lMaxSize;
    }
    /**
     * Set the maximum size of the uploaded file
     * @param paramMaxSize
     */
    public void setMaxSize(long paramMaxSize)
    {
        _lMaxSize = paramMaxSize;
    }
    
    //----------------------------------------------------------------------------
    /**
     * Returns the screen context attribute where to store the file
     * @return the screen context attribute ( or null if not set )
     */
    public String getContextAttribute()
    {
        return _sContextAttribute;
    }
    /**
     * Set the screen context attribute where to store the file
     * @param paramContextAttribute
     */
    public void setContextAttribute(String paramContextAttribute)
    {
        _sContextAttribute = paramContextAttribute;
    }
    
    //----------------------------------------------------------------------------
    /**
     * Returns the destination directory 
     * @return the directory ( or null if not set )
     */
    public String getDestDir()
    {
        return _sDestDir;
    }
    /**
     * Set the destination directory 
     * @param paramDestDir
     */
    public void setDestDir(String paramDestDir)
    {
        _sDestDir = paramDestDir;
    }
    
    //----------------------------------------------------------------------------
    /**
     * Returns the session key that will be used to store the file in the HttpSession
     * @return the key ( or null if not set )
     */
    public String getSessionKey()
    {
        return _sSessionKey;
    }
    /**
     * Set the session key that will be used to store the file in the HttpSession
     * @param paramSessionKey
     */
    public void setSessionKey(String paramSessionKey)
    {
        _sSessionKey = paramSessionKey;
    }
}