package org.objectweb.telosys.util;

import java.text.ChoiceFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;

/**
 * @author Laurent GUERIN
 *
 */
public final class NumUtil
{
    /** 
     * Private constructor 
     */
    private NumUtil()
    {
    }

    /**
     * Formats an int value in order to provide the expected number of characters
     * @param iNbr the value to format
     * @param iLength the minimum size expected 
     * @param bFixedSize true for fixed size ( cut the string if necessary )
     * @param cAppended the character to add at the beginning of the string ( e.g. '0' or ' ' )
     * @return String
     */
    public static String formatInt(final int iNbr, final int iLength, final boolean bFixedSize, final char cAppended)
    {
        String s = String.valueOf(iNbr);
        int iDiff = s.length() - iLength;
        if (iDiff > 0 && bFixedSize) // le nombre est plus long que la taille souhaite
        {
            return s.substring(0, iLength);
        }
        else if (iDiff < 0) // le nombre est moins long que la taille souhaite
        {
            iDiff = iDiff * -1;
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < iDiff; i++)
            {
                sb.append(cAppended);
            }
            return sb.append(s).toString();
        }
        return s;
    }

    /**
     * Formats an int value in order to provide the expected number of characters ( adds '0' if necessary )
     * @param iNbr :
     * @param iMinLength :
     * @return String :
     */
    public static String formatInt(final int iNbr, final int iMinLength)
    {
        return formatInt(iNbr, iMinLength, false, '0');
    }

    /**
     * Formats an int value using a standard Java format string
     * @param iValue : the value to format
     * @param sFormat : Standard Java ChoiceFormat or DecimalFormat
     * @param sLanguage : ISO Language Code ( ie "fr"=French, "en"=English ) 
     * @return : the formated value
     */
    public static String formatInt(final int iValue, final String sFormat, final String sLanguage )
    {
        if (sFormat.indexOf('|') >= 0) // Contains a CHOICE Separator
        {
            String sResult = "(?)";
            try
            {
                ChoiceFormat choiceFormat = new ChoiceFormat(sFormat);
                sResult = choiceFormat.format(iValue);
            } catch (Throwable e)
            {
                sResult = "(error)";
            }
            return sResult;
        }
        else
        {
            DecimalFormat decimalFormat = null;
            if (sLanguage != null)
            {
                //--- Create format with specific "Locale" symbols
                Locale locale = new Locale( sLanguage );
                DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
                decimalFormat = new DecimalFormat(sFormat, symbols);
            }
            else
            {
                //--- With default symbols
                decimalFormat = new DecimalFormat(sFormat);
            }
            return decimalFormat.format(iValue);
        }
    }
    
    /**
     * Formats an int value using a standard Java format string
     * @param iValue : the value to format
     * @param sFormat : Standard Java ChoiceFormat or DecimalFormat
     * @return : the formated value
     */
    public static String formatInt(final int iValue, final String sFormat )
    {
        return formatInt( iValue, sFormat, null );
    }
}